@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -525,7 +525,8 @@ processKMap (const char* filename, SInputStream& _fl, int nocomment, int reverse
   int    error;
   char    *from, *to;
   int    iscomment;
-  int    empty;
+  //SGC
+  //int    empty;
   int    index;
   int    i;
   int    currentSegment;
@@ -538,7 +539,7 @@ processKMap (const char* filename, SInputStream& _fl, int nocomment, int reverse
   char*    lineComment;
   int    lineCommentSize;
   SBMapItemVector*  retVle = 0;
-  char*     segmentArray[TOK_MAX_SEGMENTS];
+  const char*     segmentArray[TOK_MAX_SEGMENTS];
   int    getNewLine;
 
 
@@ -786,11 +787,12 @@ processKMap (const char* filename, SInputStream& _fl, int nocomment, int reverse
   if (error)
   {
     fprintf (stderr, "mytool: error- bad syntax in file '%s' at line %d\n",
-      (filename==0?"unkown":filename), lineNo);
+      (filename==0?"unknown":filename), lineNo);
     delete retVle;
     return 0;
   }
-  empty = 0;
+  // SGC
+  // empty = 0;
   for (i=0; i<segmentCount || i==0; i++)
   {
     if (retVle->maps[i]->codeSize==0) 
@@ -798,17 +800,17 @@ processKMap (const char* filename, SInputStream& _fl, int nocomment, int reverse
       if (segmentCount==0)
       {
        fprintf (stderr, "mytool: error- bad syntax in file '%s' at eof\n",
-        (filename==0?"unkown":filename));
+        (filename==0?"unknown":filename));
       }
       else if (currentSegment!=-1)
       {
        fprintf (stderr, "mytool: bad syntax (missing end?) in file '%s' at eof\n",
-        (filename==0?"unkown":filename));
+        (filename==0?"unknown":filename));
       }
       else
       {
        fprintf (stderr, "mytool: error- bad syntax in file '%s' at multipart '%s'\n",
-        (filename==0?"unkown":filename), segments[i]);
+        (filename==0?"unknown":filename), segments[i]);
       }
       delete retVle;
       return 0;
