@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -948,10 +948,8 @@ SBMap::encode (int mapIndex, const void *in, int in_size,
 {
   unsigned int  i;
   unsigned int  j;
-  int  reject;
   SBMapItem::SFound  result;
 
-  reject = SS_REJECT;
   if (status!=true || maps ==0 || mapIndex >= mapSize) return SS_REJECT;
 
   SBMapItem* map = maps[mapIndex];
@@ -2500,7 +2498,8 @@ SBMap::serialize (SOutputStream&  is, SFileFormat _format)
   // magicNN
   w = write1 (_fd, (const unsigned char*) magicNN, sizeof (magicNN), _format, FT_MAGIC, SS_LAST);
   
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Name
   if (name==0)
@@ -2514,26 +2513,31 @@ SBMap::serialize (SOutputStream&  is, SFileFormat _format)
   {
     w = write1 (_fd, name, 32, _format, FT_NAME, SS_LAST);
   }
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Comment size
   w = write32 (_fd, (SS_WORD32) commentSize, _format, FT_IGNORE, SS_NORMAL);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Comment
   if (commentSize>0)
   {
     w = write1 (_fd, comment, commentSize, _format, FT_COMMENT, SS_LAST);
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
   }
 
   // Map type
   w = write32 (_fd, (SS_WORD32) mapType, _format, FT_MAP_TYPE, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Map size
   w = write32 (_fd, (SS_WORD32) mapSize, _format, FT_IGNORE, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // SS_WORD32 indeces = 16 + 32 + 4 + commentSize + 4 + 4 + 4 * mapSize + 4;
   SS_WORD32 indeces = 0;
@@ -2545,19 +2549,22 @@ SBMap::serialize (SOutputStream&  is, SFileFormat _format)
     maps[i]->convertFromBumap();
     // size
     w = write32 (_fd, (SS_WORD32) indeces, _format, FT_IGNORE,  SS_NORMAL);
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
 
     indeces += maps[i]->getSerializeSize();
   }
   // Last index
   w = write32 (_fd, (SS_WORD32) indeces, _format, FT_IGNORE, SS_NORMAL);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Now really serialize 'em one by one.
   for (i=0; i<mapSize; i++)
   {
     w = maps[i]->serialize (_fd, _format, (int) (i==mapSize-1));
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
     // Stupidity check.
     if (w != maps[i]->getSerializeSize())
     {
@@ -2600,7 +2607,8 @@ SBMap::serializeUMAP (SString* _fd, SFileFormat _format)
   // magicNN
   w = write1 (_fd, (const unsigned char*) magicBM, sizeof (magicBM), _format, FT_MAGIC, SS_LAST);
   
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Name
   if (name==0)
@@ -2618,13 +2626,15 @@ SBMap::serializeUMAP (SString* _fd, SFileFormat _format)
   // Comment size
   w = write16 (_fd, (SS_WORD16) commentSize+(16+32+2), _format, FT_IGNORE, SS_NORMAL);
 
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Comment
   if (commentSize>0)
   {
     w = write1 (_fd, comment, commentSize, _format, FT_COMMENT, SS_LAST);
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
   }
 
 
@@ -2975,40 +2985,48 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
 
   // Name
   w = write1 (_fd, name, 32, _format, FT_SECTION, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Comment size
   w = write32 (_fd, (SS_WORD32) commentSize, _format, FT_IGNORE, SS_NORMAL);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Comment
   w = write1 (_fd, comment, commentSize, _format, FT_COMMENT, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // decode/encode
   out = encode;
   w = write8 (_fd, out, _format, FT_IS_ENCODE, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
   
   // Input word size.
   out = inWordSize;
   w = write8 (_fd, out, _format, FT_SIZE_FROM, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Output word size.
   out = outWordSize;
   w = write8 (_fd, out, _format, FT_SIZE_TO, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Input length indicator size.
   out = inByteLength;
   w = write8 (_fd, out, _format, FT_LENGTH_FROM, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Output length indicator size.
   out = outByteLength;
   w = write8 (_fd, out, _format, FT_LENGTH_TO, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Pointer to a state machine.
   // First we need to know our own length...
@@ -3018,23 +3036,27 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
   if (stateMachineSize >0 && stateMachine != 0)
   {
     w = write32 (_fd, lastIndex, _format, FT_IGNORE, SS_LAST);
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
   }
   else
   {
     w = write32 (_fd, 0, _format, FT_IGNORE, SS_LAST);
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
   }
 
   // Spare...
   w = write32 (_fd, 0, _format, FT_IGNORE, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   // Write out the array of indeces.
   // First the size.
 
   w = write32 (_fd, codeSize, _format, FT_IGNORE, SS_NORMAL);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   const unsigned char* mp;
   fullIndex=0;
@@ -3047,12 +3069,14 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
     lastIndex = fullIndex;
     
     w = write32 (_fd, lastIndex, _format, FT_IGNORE, SS_LAST);
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
     fullIndex += getLength(i);
   }
   
   w = write32 (_fd, baseSize, _format, FT_IGNORE, SS_LAST);
-  if (w<0) return -1; index += w;
+  if (w<0) return -1; 
+  index += w;
 
   //* Write out the array itself
   for (i=0; i<codeSize; i++)
@@ -3063,7 +3087,8 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
     {
       w=writeCodeArea(_fd, i, _format,
         (_last && i==codeSize-1) ? SS_EOF : SS_LAST);
-      if (w<0) return -1; index += w;
+      if (w<0) return -1;
+      index += w;
       if (w != (int) getLength(i))
       {
         fprintf (stderr, "internal error\n");
@@ -3073,7 +3098,8 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
     else
     {
       w=writeCodeArea(_fd, i, _format, SS_LAST);
-      if (w<0) return -1; index += w;
+      if (w<0) return -1; 
+      index += w;
       if (w != (int) getLength(i))
       {
         fprintf (stderr, "internal error\n");
@@ -3089,12 +3115,14 @@ SBMapItem::serialize (SString* _fd, SFileFormat _format, int _last)
     w = write32 (_fd, stateMachineSize, 
         _format, FT_IGNORE, SS_LAST);
 
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
     w = write1 (_fd, stateMachine, 
       stateMachineSize * 4 * 16, _format, FT_IGNORE, 
       (_last) ? SS_EOF : SS_LAST);
 
-    if (w<0) return -1; index += w;
+    if (w<0) return -1; 
+    index += w;
   }
   return index;
 }
@@ -3783,7 +3811,7 @@ SBMapItem::getDecoderMap (SStringVector* key, SStringVector* value,
      SV_UCS4 ucs4v;
      for (unsigned int i=0; i<vlesize; i=i+(1<<outWordSize))
      {
-       SS_UCS4 ucs4;
+       SS_UCS4 ucs4 = 0;
        switch (outWordSize)
        {
        case 0:
@@ -3832,11 +3860,11 @@ SBMapItem::getLinearPosition (SS_UCS4 key)
   switch (inWordSize)
   {
   case 0:
-    if (key>0xff) { delete bkey; return 0;}
+    if (key>0xff) { delete[] bkey; return 0;}
     bkey[0] = key&0xff;
     break;
   case 1:
-    if (key>0xffff)  { delete bkey; return 0;}
+    if (key>0xffff)  { delete[] bkey; return 0;}
     bkey[1] = key&0xff;
     bkey[0] = (key>>8)&0xff;
     break;
@@ -3847,11 +3875,11 @@ SBMapItem::getLinearPosition (SS_UCS4 key)
     bkey[0] = (key>>24)&0xff;
     break;
   default:
-    delete bkey;
+    delete[] bkey;
     return 0;
   }
   unsigned int from = nextSorted(bkey, bkeySize);
-  delete bkey;
+  delete[] bkey;
   if (from ==0) return 0;
   /* nextSorted returns next element  - the insertion point */
   return (from-1);
