@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
- 
+#define SHOW_BUGS 0 
 
 #include "SEvent.h"
 #include "SEventBSD.h"
@@ -61,8 +61,8 @@ WSADATA  wsaDataStruct;
 
 
 
-int sEventBSDSignal =0;
-void initSignals (bool init);
+static int sEventBSDSignal =0;
+static void initSignals (bool init);
 
 static long substract (const struct timeval& from, const struct timeval& what);
 
@@ -93,10 +93,10 @@ int initsockets(bool init);
 int selectHack (int readSize, fd_set *ro, int writeSize, fd_set *wo,
   int exceptSize, fd_set *ex, struct timeval* t);
 
-int nonblockHack (long id);
+static int nonblockHack (long id);
 
 /**
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-04-23
  * This is the default unix event handler implementation.
  */
@@ -156,7 +156,10 @@ SEventBSD::addTimer (STimer* tim, SEventTarget* t)
 {
   if (sourceHashtable.get (tim))
   {
-    fprintf (stderr, "attempted to add save event source (timer) twice.\n");
+// SGC BUG.
+#if SHOW_BUGS
+    fprintf (stderr, "attempted to add same event source (timer) twice.\n");
+#endif
     return;
   }
   SEventSource* s = tim;
@@ -213,7 +216,9 @@ SEventBSD::addServer (SServerStream* st, SEventTarget* t)
 {
   if (sourceHashtable.get (st))
   {
+#if SHOW_BUGS
     fprintf (stderr, "attempted to add save event source (server) twice.\n");
+#endif
     return;
   }
   SEventSource* s = st;
@@ -241,7 +246,9 @@ SEventBSD::addInput (SInputStream* ins, SEventTarget* t)
 {
   if (sourceHashtable.get (ins))
   {
-    fprintf (stderr, "attempted to add save event source (input) twice.\n");
+#if SHOW_BUGS
+    fprintf (stderr, "attempted to add same event source (input) twice.\n");
+#endif
     return;
   }
   SEventSource* s = ins;
@@ -269,7 +276,9 @@ SEventBSD::addJob (SJob* sjob, SEventTarget* t)
 {
   if (sourceHashtable.get (sjob))
   {
-    fprintf (stderr, "attempted to add save event source (input) twice.\n");
+#if SHOW_BUGS
+    fprintf (stderr, "attempted to add same event source (job) twice.\n");
+#endif
     return;
   }
   SEventSource* s = sjob;
@@ -760,16 +769,16 @@ SEventBSD::next()
      {
         remove (c);
         t->error(s);
-        delete buffer;
+        delete[] buffer;
         break;
      }
      else if (!sendData(c, buffer, n))
      {
        remove (c);
-       delete buffer;
+       delete[] buffer;
        break;
      }
-     delete buffer;
+     delete[] buffer;
      FD_CLR (fd, &ro);
    }
   }
@@ -1035,7 +1044,7 @@ initsockets(bool init)
 #endif
 
 #ifdef USE_WINAPI
-void
+static void
 initSignals(bool init)
 {
 }
@@ -1047,7 +1056,7 @@ extern "C"
   static  void signalHandler (int signum);
 };
 
-void
+static void
 initSignals(bool init)
 {
   static bool inited = false;
