@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -63,6 +63,21 @@ STextDataEvent::setRemaining (const STextIndex& _remaining)
 STextDataEvent::~STextDataEvent ()
 {
 }
+unsigned int
+STextData::softSize (unsigned int line) const
+{
+  if (line >= size()) return 0;
+  const SParagraph* l = lines[line];
+  return l->softSize();
+}
+
+SS_UCS4
+STextData::softCharAt (unsigned int line, unsigned int pos) const
+{
+  if (line >= size()) return 0;
+  const SParagraph* l = lines[line];
+  return l->softCharAt(pos);
+}
 
 /**
  * Clear all fields.
@@ -117,6 +132,8 @@ STextDataEvent::add (const STextDataEvent& event)
 STextData::STextData (void)
 {
   embedding = SS_EmbedNone;
+  listener = 0;
+  lineTracker = 0;
 }
 
 /**
@@ -126,6 +143,8 @@ STextData::STextData (void)
 STextData::STextData (const SString& utf8)
 {
   embedding = SS_EmbedNone;
+  listener = 0;
+  lineTracker = 0;
   insert (utf8);
 }
 
@@ -145,6 +164,7 @@ STextData::operator = (const STextData & data)
     p->underline (false);
     p->select (false);
     lines.append (p);
+    if (lineTracker) lineTracker->lineInserted (this, lines.size()-1);
   }
   return *this;
 }
@@ -286,6 +306,16 @@ STextData::glyphAt (const STextIndex& index) const
 {
   return (*lines[index.line])[index.index];
 }
+/**
+ * Get a glyph at a certain position
+ * @param index is the glyph index
+ * @return a reference to the glyph.
+ */
+const SGlyph*
+STextData::peekGlyphAt (const STextIndex& index) const
+{
+  return &(*lines[index.line])[index.index];
+}
 
 /**
  * move the insertion point.
@@ -349,6 +379,7 @@ STextData::setText (const SV_UCS4& ucs4)
       break;
     }
     lines.append (gl);
+    if (lineTracker) lineTracker->lineInserted (this, lines.size()-1);
   } 
   STextDataEvent nevt (STextIndex(0,0));
   event.add (nevt);
@@ -364,16 +395,19 @@ STextData::insert (const SGlyph& glyph)
 
   if (size() <= textIndex.line)
   {
-     SParagraph* p = new SParagraph();
-     CHECK_NEW (p);
-     p->setEmbedding (embedding);
-     lines.append (p);
+    SParagraph* p = new SParagraph();
+    CHECK_NEW (p);
+    p->setEmbedding (embedding);
+    lines.append (p);
+    if (lineTracker) lineTracker->lineInserted (this, lines.size()-1);
   }
 
   SParagraph* line = lines[textIndex.line];
 
   line->clearChange();
   line->insert (textIndex.index, glyph);
+  if (lineTracker) lineTracker->lineChanged (this, textIndex.line);
+
   STextDataEvent nevt (STextIndex(textIndex.line, line->getChangeStart())); 
   STextIndex end(textIndex.line, line->getChangeEnd());
 
@@ -387,8 +421,10 @@ STextData::insert (const SGlyph& glyph)
       SParagraph *newLine = new SParagraph (*line);
       newLine->remove (0, textIndex.index);
       line->truncate (textIndex.index);
+      if (lineTracker) lineTracker->lineChanged (this, textIndex.line);
+
       lines.insert (textIndex.line+1, newLine);
-      
+      if (lineTracker) lineTracker->lineInserted (this, textIndex.line+1);
 
       textIndex.index = 0;
       textIndex.line = textIndex.line + 1;
@@ -442,6 +478,7 @@ STextData::insert (const SString& utf8)
       CHECK_NEW (p);
       p->setEmbedding(embedding);
       lines.insert (textIndex.line, p);
+      if (lineTracker) lineTracker->lineInserted (this, textIndex.line);
       textIndex.line++;
       setMaxLimits (&nevt, textIndex);
       event.add (nevt);
@@ -483,6 +520,7 @@ STextData::remove (const STextIndex& index)
      {
         if (isProperLine(i)) join = true;
         line->clear();
+        if (lineTracker) lineTracker->lineChanged (this, i);
      }
      else for (unsigned int j=begin; j<end; j++)
      {
@@ -493,6 +531,7 @@ STextData::remove (const STextIndex& index)
           if (g.isEOP()) join = true;
         }
         line->remove (begin);
+        if (lineTracker) lineTracker->lineChanged (this, i);
      }
   }
   unsigned int lr = 0;
@@ -505,6 +544,7 @@ STextData::remove (const STextIndex& index)
      {
        lines.remove (i-lr);
        delete line;
+       if (lineTracker) lineTracker->lineRemoved (this, i-lr);
        lr++;
      }
   }
@@ -526,11 +566,13 @@ STextData::remove (const STextIndex& index)
       {
         lines.remove (textIndex.line);
         delete line;
+        if (lineTracker) lineTracker->lineRemoved (this, textIndex.line);
       }
       return;
     }
     SParagraph* gl = lines[textIndex.line+1];
     lines.remove (textIndex.line+1);
+    if (lineTracker) lineTracker->lineRemoved (this, textIndex.line+1);
 
     STextIndex sv = textIndex;
     for (i=0; i<gl->size(); i++)
@@ -547,6 +589,7 @@ STextData::remove (const STextIndex& index)
     {
       lines.remove (textIndex.line);
       delete line;
+      if (lineTracker) lineTracker->lineRemoved (this, textIndex.line);
     }
   }
 }
@@ -713,6 +756,7 @@ STextData::setParagraphSeparator (SS_ParaSep sep)
 void
 STextData::underline (const STextIndex& index, bool is)
 {
+
   if (size() == 0) return;
 
   STextIndex newIndex = reorder (index);
@@ -763,6 +807,10 @@ STextData::clear ()
      delete lines[i];
   }
   lines.clear();
+  if (lineTracker)
+  {
+    lineTracker->lineRemoved (this, 0);
+  }
   textIndex = STextIndex (0, 0);
   STextDataEvent nevt (STextIndex (0, 0));
   nevt.remaining = STextIndex (0, 0);
@@ -801,7 +849,14 @@ STextData::size(unsigned int line) const
 {
   if (line >= size()) return 0;
   const SParagraph* l = lines[line];
-  return l->size();
+  // This will expand the line
+  //bool exp = l->isExpanded ();
+  unsigned int ret = l->size();
+  // We are not const
+  //STextData* th = (STextData*) this;
+  // It is futile to track expanded lines in this class
+  //if (!exp && lineTracker) th->lineTracker->lineExpanded (th, line);
+  return ret;
 }
 
 /**
@@ -869,7 +924,7 @@ STextData::getTextIndex(const STextIndex& base, int charOffset, bool logical) co
   }
 
   /* set remaining */
-  ret.index =  (aco > ret.index) ? 0 : ret.index = ret.index - aco;
+  ret.index =  (aco > ret.index) ? 0 : ret.index - aco;
 
   /* as ret.index contains the remaining. we need to normalize it back  */
   if (logical)
@@ -906,6 +961,16 @@ STextData::addTextDataListener (STextDataListener* _listener)
   listener = _listener;
 }
 
+/**
+ * Add a line tracker that tracks our line array.
+ * TODO: now it only sets it.
+ */
+void
+STextData::addLineTracker (SLineTracker* lt)
+{
+  lineTracker = lt;
+}
+
 /**
  * Set the maximum limits.
  * @param evt -> remainingLine and remainingPosition will be set
@@ -1009,6 +1074,14 @@ STextData::isNumber (const STextIndex& index) const
   return glyphAt (index).isNumber();
 }
 
+/**
+ * Used in syntax higlighting.
+ */
+bool
+STextData::isLetter (const STextIndex& index) const
+{
+  return glyphAt (index).isLetter();
+}
 
 /**
  * return true if character is a target for select. Addition by Maarten van Gompel <proycon@anaproy.homeip.net>
@@ -1019,6 +1092,14 @@ STextData::isDelimiter (const STextIndex& index) const
   return glyphAt (index).isDelimiter();
 }
 
+// previously we used isDelimiter to decide weather the word can be broken
+// at this poing. Now I added CJK and KANA because they can wrap any time.
+bool
+STextData::canWrap (const STextIndex& index) const
+{
+  return glyphAt (index).canWrap();
+}
+
 unsigned int
 STextData::properSize(unsigned int line) const
 {
