@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -39,7 +39,8 @@
 SFileDialog::SFileDialog (void)
 {
   setFrameListener (this);
-  hiddenButton = new SButton (translate ("Show Hidden"));
+  hiddenButton = new SButton (translate ("Show Hidden"), 
+        SIconFactory::getIcon("CircleOff"));
   hiddenButton->setButtonListener (this);
   hiddenButton->setAlignment (SD_Center);
   hiddenButton->setIcon(SIconFactory::getIcon("CircleOff"));
@@ -120,7 +121,6 @@ SFileDialog::SFileDialog (void)
 bool
 SFileDialog::setFileType (const SString& _fileType)
 {
-  /* add this filetype */
   bool exists = true;
   if (!fileTypeListBox->selectText (_fileType))
   {
@@ -150,6 +150,11 @@ SFileDialog::setApplicationImage (const SImage& image)
 void
 SFileDialog::fileTypeChanged (const SString& _filetype)
 {
+  if (_filetype == "utf-8") {
+    fileTypeButton->setIcon (SIconFactory::getIcon("Best"));
+  } else {
+    fileTypeButton->setIcon (SIconFactory::getIcon("BestEmpty"));
+  }
   //SString a="[";
   SString a;
   a.append(_filetype);
@@ -212,6 +217,8 @@ SFileDialog::rereadDir()
   }
   folderListBox->setText(combedDirs);
   folderNameLabel->setText (currentDirectory.getUnixName ());
+  // This bug has been with us for some time now :)
+  fileListBox->selectText (fileTextEdit->getText());
 }
 
 void
@@ -433,7 +440,7 @@ SFileDialog::recalc ()
 
   setMinimumSize (preferredSize);
 
-  resize (preferredSize);
+  resize (SDimension (preferredSize.width, preferredSize.height));
 
 }
 
@@ -474,6 +481,7 @@ SFileDialog::getInput (const SString& titleString,
   fileFilter = "*";
   filterTextEdit->setText (fileFilter);
   rereadDir ();
+  fileListBox->selectText (fileTextEdit->getText());
   center();
   isCancel = true;
   isFocused = false;
@@ -488,7 +496,7 @@ SFileDialog::getInput (const SString& titleString,
  * buttons call this
  */
 void
-SFileDialog::buttonPressed (void* source, const SAccelerator* acc)
+SFileDialog::buttonPressedAccel (void* source, const SAccelerator* acc)
 {
   if (source == createButton)
   {
@@ -499,10 +507,10 @@ SFileDialog::buttonPressed (void* source, const SAccelerator* acc)
     bool hasSlash = false;
     for (ck=0; ck<s.size(); ck++)
     {
-       if (s[ck] <= ' ') break;
+       if ((int) (unsigned char) s[ck] <= ' ') break;
        if (s[ck] == '/') hasSlash = true;
     }
-    if (ck==0 || ck<s.size())
+    if (ck==0 || ck+1 == s.size())
     {
       textDialog->getInput ("Error", 
            translate ("Will not create\nspecified folder\n"),
@@ -607,7 +615,9 @@ SFileDialog::itemSelected (void* source, const SAccelerator* acc)
   if (source == fileListBox->textList)
   {
      SString s = fileListBox->textList->getLastSelectedText();
-     fileTextEdit->setText (s);
+     if (s != fileTextEdit->getText()) {
+         fileTextEdit->setText (s);
+     }
      return;
   }
   if (source == fileTypeListBox->textList)
@@ -636,6 +646,16 @@ SFileDialog::textEntered (void *source)
     return;
   }
 }
+
+void
+SFileDialog::textChanged (void *source)
+{
+  if (source == fileTextEdit)
+  {
+     fileListBox->selectText (fileTextEdit->getText());
+  }
+}
+
 void
 SFileDialog::filterChanged ()
 {
@@ -742,11 +762,11 @@ SFileDialog::setBackground (const SColor& bg)
 }
 
 void
-SFileDialog::setForeground (const SColor& fg)
+SFileDialog::setForeground (const SColor& lrfg, const SColor& rlfg)
 {
-  SFrame::setForeground (fg);
-  setLabelForeground (fg);
-  setTitleForeground (fg);
+  SFrame::setForeground (lrfg, rlfg);
+  setLabelForeground (lrfg);
+  setTitleForeground (lrfg);
 }
 
 void
@@ -802,12 +822,13 @@ SFileDialog::setFontSize (double fontSize)
   recalc();
 }
 /**
- * keyboard focus lost.
+ * keyboard focus gained.
  */
-void 
+bool 
 SFileDialog::gainedKeyboardFocus (SWindow* w)
 {
-  if (isFocused) return;
+  if (isFocused) return true;
   isFocused = true;
   fileTextEdit->setFocus();
+  return true;
 }
