@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -38,7 +38,7 @@ public:
   ~SFrame ();
   SFrameListener* setFrameListener (SFrameListener *l);
   SFrameListener* removeFrameListener ();
-  virtual void redraw(SWindow* w, int x, int y,
+  virtual void redraw(SCanvas* w, int x, int y,
      unsigned int width ,unsigned int height);
   virtual bool windowClose (SWindow* w);
   virtual void setApplicationImage (const SImage& image);
