@@ -1,7 +1,7 @@
-/** 
+/* 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -20,9 +20,17 @@
 #include "swidget/STextEdit.h"
 #include "stoolkit/SEncoder.h"
 
+#ifdef  USE_OSX
+# define SS_NO_STATUS_UNDER true
+#else
+# define SS_NO_STATUS_UNDER false
+#endif
+
 SDClick::SDClick (void)
 {
+//fprintf (stderr, "DClick create\n");
   clickCount = 0;
+  timeoutCount = 0;
   timer = 0;
 }
 
@@ -37,6 +45,7 @@ SDClick::start (unsigned int millisec)
   if (timer == 0)
   {
     timer = STimer::newTimer(300, this);
+    // 1 click is zero.
     clickCount = 0;
   }
   else
@@ -51,13 +60,16 @@ SDClick::start (unsigned int millisec)
 bool
 SDClick::timeout (const SEventSource* s)
 {
+  timeoutCount++;
   if (timer == 0)
   {
     fprintf (stderr, "strange. timeout..\n");
     return false;
   }
-  if (clickCount == 1) return true; /* again */
+  // triple clicks have twice the timeout.
+  if (timeoutCount < 2 && clickCount == 1) return true; /* again */
   clickCount = 0;
+  timeoutCount = 0;
   delete timer;
   timer = 0;
   return false; /* don't call me again */
@@ -70,6 +82,8 @@ SDClick::timeout (const SEventSource* s)
  */
 STextEdit::STextEdit (void) : textView ("")
 {
+  autoClipboard = true;
+  preEditSize = 0;
   currentHistorySize = 0;
   historySize = 0;
   sliderListener = 0;
@@ -118,6 +132,8 @@ STextEdit::~STextEdit ()
  */
 STextEdit::STextEdit (const SString& utf8) : textView (utf8)
 {
+  autoClipboard = true;
+  preEditSize = 0;
   currentHistorySize = 0;
   historySize = 0;
   sliderListener = 0;
@@ -262,7 +278,7 @@ STextEdit::isMultiline () const
  * @param canvas is where we redraw this.
  */
 void
-STextEdit::redraw (SWindow *w, int x, int y, unsigned int width, unsigned int height)
+STextEdit::redraw (SCanvas *w, int x, int y, unsigned int width, unsigned int height)
 {
   clip (false);
   border.redraw (w, x, y, width, height);
@@ -275,7 +291,7 @@ STextEdit::redraw (SWindow *w, int x, int y, unsigned int width, unsigned int he
   {
     notifySlider();
     updateCaretLocation ();
-    redraw();
+    redrawInternal();
   }
   /*
    * We need this hack because when resizing, we may not 
@@ -342,7 +358,7 @@ fprintf (stderr, "try: %*.*s %*.*s %*.*s\n",
         && shift == true && s == " ") 
   {
     //setInputMethod (SS_KINPUT2_IM);
-    if (!window->startInputMethod ("_JAPANESE_CONVERSION",getProperties()))
+    if (!window->startInputMethod ("_JAPANESE_CONVERSION",getProperties(),this))
     {
       fprintf (stderr, "Can not start kinput2 (_JAPANESE_CONVERSION) input method.\n");
     }
@@ -374,12 +390,15 @@ void
 STextEdit::buttonPressed (SWindow * w, int button, int x, int y)
 {
   if (yuditInput) yuditInput->clear();
+  window->commitInputMethod();
   window->getKeyboardFocus();
   SCursorIndex textIndex = textView.getCursorIndex (SLocation (x, y));
   setFocus ();
-
-  /* double click check */
-  clicks.start (500);    
+  // not scroll
+  if (button != 4 && button != 3) {
+    /* double click check */
+    clicks.start (990);    
+  }
   editor.buttonPressed (button, textIndex);
 }
 
@@ -390,6 +409,7 @@ void
 STextEdit::buttonReleased (SWindow * w, int button, int x, int y)
 {
   if (yuditInput) yuditInput->clear();
+  window->commitInputMethod();
   editor.buttonReleased (button, textView.getCursorIndex (SLocation (x, y)));
   if (clicks.clickCount == 1)
   {
@@ -405,6 +425,7 @@ void
 STextEdit::buttonDragged (SWindow * w, int button, int x, int y)
 {
   if (yuditInput) yuditInput->clear();
+  window->commitInputMethod();
   editor.buttonDragged (button, textView.getCursorIndex (SLocation (x, y)));
 }
 
@@ -442,10 +463,14 @@ STextEdit::lostClipSelection (SWindow* w)
 /**
  * keyboard focus lost.
  */
-void 
+bool 
 STextEdit::gainedKeyboardFocus (SWindow* w)
 {
   focused = true;
+/*
+  fprintf (stderr, "STextEdit::gainedKeyboardFocus %lx editable %d\n",
+        (unsigned long) this, editable);
+*/
   if (editable)
   {
     caret.animate (true);
@@ -454,8 +479,10 @@ STextEdit::gainedKeyboardFocus (SWindow* w)
   {
     listeners[i]->focusChanged (this, true);
   }
+  return true;
 }
 
+
 /**
  * Resize the component
  * @param d is the new size
@@ -549,8 +576,16 @@ STextEdit::scrollVertical (int value, bool notify)
   SLocation v = textView.getViewPort();
   v.y = v.y + value;
   textView.setViewPort (v);
+  /* redraw whole thing */
+  if (window->isDoubleBufferEnabled ())
+  {
+    window->redraw (true,
+       (int) border.getBorderSize().width ,
+       (int) border.getBorderSize().height,
+       ewidth, eheight);
+  }
   /* outof range. */
-  if ((value < 0 && -value >= eheight) || (value > 0 && value >= eheight))
+  else if ((value < 0 && -value >= eheight) || (value > 0 && value >= eheight))
   {
     window->redraw (true,
        (int) border.getBorderSize().width ,
@@ -609,8 +644,16 @@ STextEdit::scrollHorizontal (int value, bool notify)
   int ewidth = (int)size.width - (int)2 * border.getBorderSize().width 
        + 2 * SS_LEFT_MARGIN;
   if (ewidth <= 0) return;
+  /* redraw whole thing */
+  if (window->isDoubleBufferEnabled ())
+  {
+    window->redraw (true,
+       (int) border.getBorderSize().width ,
+       (int) border.getBorderSize().height,
+       ewidth, eheight);
+  }
   /* outof range. */
-  if ((value < 0 && -value >= ewidth) || (value > 0 && value >= ewidth))
+  else if ((value < 0 && -value >= ewidth) || (value > 0 && value >= ewidth))
   {
     window->redraw (true,
        (int) border.getBorderSize().width ,
@@ -868,7 +911,7 @@ STextEdit::setFont (const SString& fnt, double _size)
   caret.resize (SDimension(textView.lineHeight, textView.lineHeight));
   resized = true;
   updateCaretLocation ();
-  redraw ();
+  redrawInternal ();
 }
 
 /**
@@ -883,14 +926,14 @@ STextEdit::setFontSize (double _size)
   caret.resize (SDimension(textView.lineHeight, textView.lineHeight));
   resized = true;
   updateCaretLocation ();
-  redraw ();
+  redrawInternal ();
 }
 
 /**
  * redraw the whole thing 
  */
 void
-STextEdit::redraw ()
+STextEdit::redrawInternal ()
 {
   int eheight = (int)size.height - (int)2 * border.getBorderSize().height;
   if (eheight <= 0) return;
@@ -938,6 +981,8 @@ STextEdit::setInputMethod (const SString& _im)
   statusHeight = 0;
   SDimension d = getSize();
 
+  window->commitInputMethod();
+
   if (yuditInput)
   {
     yuditInput->clear();
@@ -946,20 +991,26 @@ STextEdit::setInputMethod (const SString& _im)
   }
   else
   {
+    window->commitInputMethod();
   }
   if (inputMethod == SS_DEFAULT_IM)
   {
     if (oldH != statusHeight) resizeTextView (true);
-    window->startInputMethod ("x-ascii", SProperties());
+    window->startInputMethod ("x-ascii", SProperties(),this);
   }
   else if (inputMethod == SS_KINPUT2_IM)
   {
-    statusHeight = (textView.lineHeight < 24) ? 24: textView.lineHeight;
-    resizeTextView (true);
-    if (!window->startInputMethod ("_JAPANESE_CONVERSION",getProperties()))
+    if (!SS_NO_STATUS_UNDER) {
+        statusHeight = (textView.lineHeight < 24) ? 24: textView.lineHeight;
+        resizeTextView (true);
+    }
+    if (!window->startInputMethod ("_JAPANESE_CONVERSION",getProperties(),this))
     {
       fprintf (stderr, "Can not start kinput2 (_JAPANESE_CONVERSION) input method.\n");
     }
+    if (SS_NO_STATUS_UNDER) {
+        return;
+    }
     if (window->getInputMethod()=="x-utf-8" 
      || window->getInputMethod() == "x-ascii" 
      || window->getInputMethod() == ""
@@ -973,12 +1024,17 @@ STextEdit::setInputMethod (const SString& _im)
   }
   else if (inputMethod == SS_WINDOWS_IM)
   {
-    statusHeight = (textView.lineHeight < 24) ? 24: textView.lineHeight;
-    resizeTextView (true);
-    if (!window->startInputMethod ("_JAPANESE_CONVERSION",getProperties()))
+    if (!SS_NO_STATUS_UNDER) {
+        statusHeight = (textView.lineHeight < 24) ? 24: textView.lineHeight;
+        resizeTextView (true);
+    }
+    if (!window->startInputMethod ("_JAPANESE_CONVERSION",getProperties(),this))
     {
       fprintf (stderr, "Can not start Window input method.\n");
     }
+    if (SS_NO_STATUS_UNDER) {
+        return;
+    }
     if (window->getInputMethod()=="x-utf-8" 
      || window->getInputMethod() == "x-ascii" 
      || window->getInputMethod() == ""
@@ -995,16 +1051,23 @@ STextEdit::setInputMethod (const SString& _im)
       && (inputMethod[0] == 'x' || inputMethod[0] == 'X') 
       && (inputMethod[1] == '-' || inputMethod[1] == '-') )
   {
-    statusHeight = (textView.lineHeight < 24) ? 24: textView.lineHeight;
-    resizeTextView (true);
-    if (!window->startInputMethod (inputMethod, getProperties()))
+    if (!SS_NO_STATUS_UNDER) {
+        statusHeight = (textView.lineHeight < 24) ? 24: textView.lineHeight;
+        resizeTextView (true);
+    }
+    if (!window->startInputMethod (inputMethod, getProperties(),this))
     {
       fprintf (stderr, "Can not start %*.*s input method.\n", 
           SSARGS(inputMethod));
     }
+    
+    if (SS_NO_STATUS_UNDER) {
+        return;
+    }
     if (window->getInputMethod()=="x-utf-8" 
      || window->getInputMethod() == "x-ascii" 
      || window->getInputMethod() == ""
+     || window->hasStatusArea() == false
      || ximProperties.get ("InputStyle")  == 0
      || (ximProperties["InputStyle"] != "preedit-under-status-under"
          && ximProperties["InputStyle"] != "preedit-over-status-under")) 
@@ -1016,7 +1079,7 @@ STextEdit::setInputMethod (const SString& _im)
   else
   {
     if (oldH != statusHeight) resizeTextView (true);
-    window->startInputMethod ("x-ascii", SProperties());
+    window->startInputMethod ("x-ascii", SProperties(),this);
     yuditInput = new SYuditInput (inputMethod, this, &editor);
   }
 }
@@ -1050,7 +1113,8 @@ STextEdit::insertClipboardText ()
   for (unsigned int i=0; i<s4.size(); i++)
   {
     SS_UCS4 u = s4[i];
-    if (u==SD_CD_CR || u==SD_CD_LF || u==SD_CD_LS || u==SD_CD_PS) break;
+    if (u==SD_CD_CR || u==SD_CD_LF || u==SD_CD_LS 
+       || u==SD_CD_PS || u==SD_CD_FF) break;
     vle.append ((SS_UCS4) u);
   }
   textView.textData.insert (oc.encode(vle));
@@ -1070,8 +1134,17 @@ STextEdit::insertClipboardText ()
  * Strictly speaking this is not and EditorIM
  */
 void
-STextEdit::clipSelection ()
+STextEdit::autoClipSelection ()
 {
+  if (autoClipboard) clipSelect();
+  
+}
+
+void
+STextEdit::clipSelect ()
+{
+  if (textView.isHideText()) return;
+
   if (startSelection == endSelection)
   {
     return;
@@ -1080,6 +1153,19 @@ STextEdit::clipSelection ()
   window->putClipUTF8 (str);
 }
 
+// Return an empty string if there is no selected text, or
+// return the text currently selected.
+SString
+STextEdit::getSelectedText ()
+{
+  if (startSelection == endSelection)
+  {
+    return SString("");
+  }
+  SString str = textView.textData.getText (startSelection, endSelection);
+  return SString (str);
+}
+
 /**
  * Dirty text may conta8ing newline characters and the line. 
  * This routine is calling insertText for multiline widget.
@@ -1109,17 +1195,23 @@ STextEdit::insertDirtyText (const SString& str, bool embed)
   int ind0 = str.find (SS_LB_DOS); 
   int ind1 = str.find (SS_LB_MAC); 
   int ind2 = str.find (SS_LB_UNIX); 
-  int ind3 = str.find (SS_LB_LS); 
+  int ind3 = str.find (SS_LB_PS); 
+  int ind4 = str.find (SS_LB_LS); 
+  int ind5 = str.find (SS_LB_FF); 
 
   if (ind0 < 0) ind0 = str.size();
   if (ind1 < 0) ind1 = str.size();
   if (ind2 < 0) ind2 = str.size();
   if (ind3 < 0) ind3 = str.size();
+  if (ind4 < 0) ind4 = str.size();
+  if (ind5 < 0) ind5 = str.size();
 
   int smallest = ind0;
   if (ind1 < smallest) smallest = ind1;
   if (ind2 < smallest) smallest = ind2;
   if (ind3 < smallest) smallest = ind3;
+  if (ind4 < smallest) smallest = ind4;
+  if (ind5 < smallest) smallest = ind5;
 
   if (smallest < (int)str.size())
   {
@@ -1180,9 +1272,9 @@ STextEdit::insertText (const SString& _str, bool embed)
     }
   }
   textView.textData.insert (str);
-  textView.textData.fireEvent();
-  /* move to the logical end of previous character */
+  textView.textData.fireEvent ();
 
+  /* move to the logical end of previous character */
   updateCaretPosition ();
   updateCaretLocation ();
 
@@ -1196,7 +1288,7 @@ STextEdit::insertText (const SString& _str, bool embed)
 void
 STextEdit::pageUp ()
 {
-  if (textView.lineHeight == 0 && !isMultiline() 
+  if ((textView.lineHeight == 0 && !isMultiline()) 
      || textView.textData.size ()==0)
   {
     return;
@@ -1220,7 +1312,7 @@ STextEdit::pageUp ()
     unsigned int h1 = textView.getDocumentHeight(); 
     if (h1 != h0)
     {
-      redraw();
+      redrawInternal();
     }
     return;
   }
@@ -1251,7 +1343,7 @@ STextEdit::pageUp ()
   unsigned int h1 = textView.getDocumentHeight(); 
   if (h1 != h0)
   {
-    redraw();
+    redrawInternal();
   }
   return;
 }
@@ -1262,7 +1354,7 @@ STextEdit::pageUp ()
 void
 STextEdit::pageDown ()
 {
-  if (textView.lineHeight == 0 && !isMultiline() 
+  if ((textView.lineHeight == 0 && !isMultiline()) 
      || textView.textData.size ()==0)
   {
     return;
@@ -1347,7 +1439,7 @@ STextEdit::caretUp ()
   unsigned int h1 = textView.getDocumentHeight(); 
   if (h1 != h0)
   {
-    redraw();
+    redrawInternal();
   }
   return;
 }
@@ -1470,7 +1562,7 @@ STextEdit::endSelect ()
   selecting = false;
   if (startSelection != endSelection)
   {
-    clipSelection();
+     autoClipSelection();
   }
 }
 
@@ -1590,7 +1682,7 @@ STextEdit::selectWord ()
     textView.textData.fireEvent ();
     textView.textData.move (index);
     /* caret did not move */
-    clipSelection();
+    autoClipSelection();
   }
 }
 
@@ -1619,7 +1711,7 @@ STextEdit::selectLine ()
   textView.textData.fireEvent ();
   /* caret did not move */
   textView.textData.move (index);
-  clipSelection();
+  autoClipSelection();
 }
 
 /**
@@ -1734,7 +1826,8 @@ STextEdit::backspace ()
   if (indexPrev == indexNow) return SString("");
   SString ret = textView.textData.getText (indexPrev);
   textView.textData.remove (indexPrev);
-  textView.textData.fireEvent();
+  // fireTextDataEvent is used now. textView.textData.fireEvent();
+  textView.textData.fireEvent ();
   fireTextChanged ();
   updateCaretPosition ();
   updateCaretLocation ();
@@ -1876,13 +1969,21 @@ STextEdit::isSelecting () const
  * Insert text from pre-edit, bypassing undo
  */
 void
-STextEdit::insertPreEditText (const SString& str)
+STextEdit::insertPreEditText (const SString& str, SPreEditor::SStyle style)
 {
   STextIndex before = textView.textData.getTextIndex ();
   /* this will move caret */
   insertDirtyText (str);
   STextIndex now = textView.textData.getTextIndex ();
-  textView.textData.underline (before);
+  switch (style)
+  {
+  case SPreEditor::Style_Selected:
+    textView.textData.select (before);
+    break;
+  case SPreEditor::Style_Default:
+  default:
+    textView.textData.underline (before);
+  }
   textView.textData.move (now);
   textView.textData.fireEvent ();
   return;
@@ -2057,6 +2158,9 @@ STextEdit::clearState()
 {
   bool had=false;
   selecting = false;
+  if (window->commitInputMethod()) {
+    had = true;
+  }
   if (yuditInput && yuditInput->clear(false))
   {
     had = true;
@@ -2064,7 +2168,7 @@ STextEdit::clearState()
   if (startSelection != endSelection)
   {
     had=true;
-    clipSelection();
+    autoClipSelection();
   }
   deselectText();
   return had;
@@ -2099,6 +2203,7 @@ void
 STextEdit::clear()
 {
   if (yuditInput) yuditInput->clear();
+  window->commitInputMethod();
   startSelection = STextIndex(0,0);
   endSelection = STextIndex(0,0);
   selecting = false;
@@ -2372,16 +2477,28 @@ STextEdit::getCaretArrowFrom ()
 }
 
 void
-STextEdit::setHighlightMode (STextView::SS_HighlightMode hlMode)
+STextEdit::setSyntax (const SString& hlMode)
 {
-  textView.setHighlightMode(hlMode);
-  redraw ();
+  textView.setSyntax(hlMode);
+  redrawInternal ();
 }
 
-STextView::SS_HighlightMode
-STextEdit::getHighlightMode () const
+const SString&
+STextEdit::getSyntax () const
 {
-  return textView.getHighlightMode();
+  return textView.getSyntax();
+}
+
+void
+STextEdit:: setSyntaxColors (const SSyntaxColors& attr)
+{
+  textView.setSyntaxColors(attr);
+}
+
+const SSyntaxColors&
+STextEdit::getSyntaxColors () const
+{
+  return textView.getSyntaxColors();
 }
 
 void
@@ -2393,7 +2510,7 @@ STextEdit::setWordWrap (bool lbm)
   notifySlider();
   setCursorIndex (xy);
   setCaretLocationVisible ();
-  redraw ();
+  redrawInternal ();
 }
 
 bool
@@ -2508,6 +2625,8 @@ STextEdit::changeDirection (SS_DR_Dir dir)
   {
     endSelect ();
   }
+  if (yuditInput) yuditInput->clear();
+  window->commitInputMethod();
   /* if we want to change the same emvedding dont do it */
   SEmbedState ec = caret.getEmbedState();
   SCursorIndex ci = caret.getCursorIndex();
@@ -2550,8 +2669,8 @@ STextEdit::changeDirection (SS_DR_Dir dir)
     deselectText();
   }
   /* just reset */
-  if (skip || (dir == SS_DR_L || dir == SS_DR_R) 
-     && startSelection == endSelection)
+  if (skip || ((dir == SS_DR_L || dir == SS_DR_R) 
+     && startSelection == endSelection))
   {
     /* pop embedding direction and return 1 if 
      * next direction is different. Also lower the 
@@ -2900,7 +3019,7 @@ STextEdit::getProperties ()
   p.put ("InputSpot", arr);
 
   /* add required and optional parameters if any */
-  if (p["InputStyle"] == SString("preedit-under-status-under"))
+  if (!SS_NO_STATUS_UNDER && p["InputStyle"] == SString("preedit-under-status-under"))
   {
     SWindow* w = getComponentWindow();
     unsigned long wid = w->getWindowID();
@@ -2931,7 +3050,7 @@ STextEdit::getProperties ()
    
   } 
   /* add required and optional parameters if any */
-  if (p["InputStyle"] == SString("preedit-over-status-under"))
+  if (!SS_NO_STATUS_UNDER && p["InputStyle"] == SString("preedit-over-status-under"))
   {
     SLocation vl = textView.getLocation();
     vl.x = vl.x - (int) SS_LEFT_MARGIN;
@@ -2972,3 +3091,124 @@ STextEdit::setXIMProperties (const SProperties& props)
 {
    ximProperties = props;
 }
+void
+STextEdit::preEditClearMarkedText () {
+    if (preEditSize == 0) {
+        preEditStartDirection = getDirection();
+        preEditStartIndex = getCursorIndex();
+        return;
+    }
+    SCursorIndex s = preEditStartIndex;
+    SCursorIndex e = preEditEndIndex;
+
+    endSelect ();
+    setCursorIndex (s);
+    eraseText (e.getTextIndex());
+    setCursorIndex (preEditStartIndex);
+    setDirection (preEditStartDirection);
+
+    preEditSize = 0;
+}
+
+void
+STextEdit::preEditInsertMarkedText(const SString& utf8, SStyle style=Style_Default) {
+    if (preEditSize == 0) {
+        preEditStartDirection = getDirection();
+        preEditStartIndex = getCursorIndex();
+    }
+    if(utf8.size() == 0) {
+        return;
+    }
+    insertPreEditText (utf8, style);
+    // FIXME figure out where the first caracter is.
+    preEditEndIndex = getCursorIndex();
+    preEditSize++;
+    return;
+}
+// Count characters as if they were UTF16
+// FIXME: SMP surrogates.
+static unsigned int countUTF16(const SV_UCS4& ucs4v) 
+{
+    unsigned int count =0;
+    for(unsigned int i=0; i<ucs4v.size(); i++) {
+        SS_UCS4 ucs4 = ucs4v[i]; 
+        if (ucs4 < 0x10000) {
+            count++;
+        } else {
+            count++;
+            count++;
+        }
+    }
+    return count;
+}
+
+SRectangle
+STextEdit::getGlyphRectangle (const STextIndex& ti) {
+  SCursorIndex ci;
+  if (ti.index == 0)
+  {
+    ci = SCursorIndex(ti.line, ti.index, true);
+  }
+  else
+  {
+    // both before.
+    ci = SCursorIndex(ti.line, ti.index, true);
+    //ci = SCursorIndex(ti.line, ti.index-1, false);
+  }
+//fprintf (stderr, "STextEdit::getGlypRectangle index %u\n", ti.index);
+  //bool lr = textView.textData.isLR(textIndex);
+  SLocation lc = textView.getCursorLocation (ci);
+  // screen top is zero.
+  //fprintf (stderr, "lc.x=%d lc.y=%d\n", lc.x, lc.y);
+  return (SRectangle(lc.x, lc.y, textView.lineAscent, textView.lineAscent));
+  //return (SRectangle(lc.x, lc.y, textView.lineHeight, textView.lineHeight));
+  //caret.move (lc);
+}
+
+// FIXME: SMP surrogates.
+// FIXME: Currently only works on LR text.
+SRectangle
+STextEdit::preEditGlyphRectangleUTF16(unsigned int charPos) {
+    SRectangle ret;
+    if (preEditSize == 0) {
+        return SRectangle(ret);
+    }
+    // Find the characterIndex.
+    SCursorIndex s = preEditStartIndex;
+    SCursorIndex e = getCursorIndex();
+    if (s.getTextIndex().line != e.getTextIndex().line) {
+        return SRectangle(ret);
+    }
+    if (s.getTextIndex().index > e.getTextIndex().index) {
+        return getGlyphRectangle (e.getTextIndex());
+        // LR is beginning.
+    }
+    STextIndex ti = s.getTextIndex();
+    if (ti.line >= textView.textData.size()) {
+        return getGlyphRectangle(ti);
+    }
+    unsigned int currentSize = 0;
+    ret = getGlyphRectangle (ti);
+    while (currentSize <= charPos) {
+        if (ti.index >= textView.textData.size(ti.line)) break;
+        if (ti.index >= e.getTextIndex().index) break; 
+        const SGlyph* glyph = textView.textData.peekGlyphAt (ti);
+        currentSize += countUTF16(glyph->getChars());
+        //currentSize += 1;
+        ret = getGlyphRectangle (ti);
+        ti.index = ti.index+1;
+    }
+    return SRectangle(ret);
+}
+
+void
+STextEdit::setAutoClipboard(bool is) 
+{
+   autoClipboard = is; 
+}
+
+void
+STextEdit::setHideText(bool is)
+{
+   textView.setHideText(is);
+}
