@@ -71,6 +71,7 @@ STextView::STextView (void)
   rlpen (SColor (0.0, 0.0, 1.0, 1.0)),
   underlineColor("red")
 {
+  isHidingText = false;
   printerPageSize = 0;
   highlightMode = "";
   isWordWrapOn = false;
@@ -95,6 +96,7 @@ STextView::STextView (const SString& utf8)
   rlpen (SColor (0.0, 0.0, 1.0, 1.0)),
   underlineColor("red")
 {
+  isHidingText = false;
   printerPageSize = 0;
   highlightMode = "";
   isWordWrapOn = false;
@@ -338,7 +340,7 @@ STextView::getCursorIndex (const SLocation& l)
   unsigned int si = textData.size(line);
   unsigned int nextExt = si;
   if (i<brk.size()) nextExt = brk[i];
-  bool endsline = false;
+  // SGC bool endsline = false;
 
   /* find the x position witihn the line */
   SCursorIndex cindex (line, currExt);
@@ -356,7 +358,7 @@ STextView::getCursorIndex (const SLocation& l)
 
      if (g.isEOP())
      {
-       endsline = true; /* never used */
+       // SGC endsline = true; 
      }
      else if (lm.x >= (int)begpos && lm.x < (int)midpos)
      {
@@ -469,21 +471,6 @@ STextView::getTextLocation (const STextIndex& textIndex, bool before)
   return SLocation(nloc, ret.y);
 }
 
-/**
- * This is coming from the SWindowListener
- * @param c is the canvas to draw on.
- * @param x is the upper lect corner.
- * @param y is the upper lect corner.
- * @param width is the width of this event.
- * @param height is the height of this event.
- */
-void
-STextView::redraw (SWindow *w, int x, int y, 
-        unsigned int width, unsigned int height)
-{
-  internalRedraw (w, x, y, width, height);
-}
-
 /**
  * This is coming from the SWindowListener
  * @param c is the canvas to draw on.
@@ -744,6 +731,11 @@ STextView::syntaxHighlight(STextIndex index, SPen* pen, bool *isError)
     {
       pen->setForeground(SColor("orange"));
     }
+    // emoji
+    else if (g.getType() == SD_CC_So) 
+    {
+      pen->setForeground(SColor("gray90"));
+    }
     else // not a letter, nor a number
     {
       pen->setForeground(SColor("CornflowerBlue"));
@@ -760,6 +752,11 @@ STextView::syntaxHighlight(STextIndex index, SPen* pen, bool *isError)
     {
       pen->setForeground(SColor("orange4"));
     }
+    // emoji
+    else if (g.getType() == SD_CC_So) 
+    {
+      pen->setForeground(SColor("gray20"));
+    }
     else // not a letter, nor a number
     {
       pen->setForeground(SColor("DeepSkyBlue4"));
@@ -816,11 +813,19 @@ STextView::drawGlyph (SCanvas* c,
   
   dm.translate (0, font.ascent ());
   dm.translate ((double)l.x, (double)l.y);
+
+  if (isHidingText){
+    SColor fg = lrpen.getForeground();
+    c->bitfill (fg, l.x, l.y, ext, lineHeight);
+    return;
+  }
+
   SPen p (lrpen);
   if (!g.isLR())
   {
     p = rlpen;
   }
+
   unsigned int explevel = g.getExplicitLevel();
   bool isError = false;
   if (g.selected)
@@ -1556,9 +1561,9 @@ STextView::wrapAndPosition (unsigned int line, SH_UINT* cache)
      pa.append (logical[i]);
   }
 
-  delete logicalBefore;
-  delete logical;
-  delete visual;
+  delete[] logicalBefore;
+  delete[] logical;
+  delete[] visual;
 
   breaks.insert(line, b);
   posAfter.insert(line, pa);
@@ -1830,3 +1835,15 @@ STextView::moveCursor (const SCursorIndex& ci, bool isup)
   }
   return SCursorIndex (res);
 }
+
+
+void
+STextView::setHideText(bool is)
+{
+    isHidingText = is;
+}
+bool
+STextView::isHideText()
+{
+    return isHidingText;
+}
