@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -30,6 +30,7 @@
 #include "stoolkit/SHashtable.h"
 #include "stoolkit/SExcept.h"
 #include "stoolkit/SUtil.h"
+#include "stoolkit/SCluster.h"
 
 static SStringVector fontSearchPath(
    "/,fonts,../fonts,/etc/fonts,/usr/share/yudit/fonts");
@@ -38,7 +39,7 @@ static SStringVector fontSearchPath(
 /* it makes a big difference in speed */
 
 /**
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-04-23
  * This is the abstract widget toolkit font package
  */
@@ -409,6 +410,7 @@ SFontImpl::guessPath (const SStringVector& files, const SString& property)
 {
   
   SStringVector outDataPath;
+
   for (unsigned int i=0; i<files.size(); i++)
   {
     SProperties p;
@@ -525,9 +527,10 @@ SFontImpl::scale (double x, double y)
  * @return true is drawn.
  */
 bool
-SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m, 
-  SS_UCS4 uch)
+SFontImpl::draw (SCanvas* canvas, const SPen& aPen, const SS_Matrix2D& m, 
+  SS_UCS4 uch, bool isLRContext, bool isSelected, bool baseOK)
 {
+  SPen pen (aPen);
   if (!delegate) return 0;
 
   SFontDelegate* d = (SFontDelegate*) delegate;;
@@ -537,6 +540,11 @@ SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
   case SFontDelegate::SE_TTF:
     if (d->u.ttf->isOK())
     {
+        if (!isSelected && d->u.ttf->getIsEmoji()) 
+        {
+          pen.setForeground (SColor("black"));
+          pen.setBackground (SColor("white"));
+        }
         if (!d->u.ttf->width (m, uch, 0)) return 0;
         long larr[8];
         larr[0] = 'T' ; larr[1] = 'T'; larr[2] = 'F';
@@ -546,6 +554,13 @@ SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
         /* FIXME: you should in fact use an instance counter. */
         key.append ((long)(sd.x0*16000.0)); /* scale-x */
         key.append ((long)(sd.y1*16000.0)); /* scale-y */
+        if (!isLRContext) key.append ("R");
+        SColor fg = pen.getForeground();
+        key.append ("fg:");
+        key.append ((char)fg.red);
+        key.append ((char)fg.green);
+        key.append ((char)fg.blue);
+        key.append ((char)fg.alpha);
         double offsetX = 0.0;
         double offsetY = 0.0;
         /* for better positioning of diacritical marks */
@@ -555,13 +570,30 @@ SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
         //key.append ((long)(offsetY*16000.0)); 
         sd.t0  += offsetX;
         sd.t1  += offsetY;
-        /* reverse coordinates Y, negative m.t1 */
-        if (!canvas->newpath (m.t0+offsetX, m.t1+offsetY, key))
+//fprintf (stderr, "draw %x baseOK=%d\n", uch, (int)baseOK);
+        if (!isSelected && d->u.ttf->getIsEmoji() && !baseOK) 
         {
-          /* not cached yet */
-          d->u.ttf->draw (canvas, sd, uch);
+          double width = 1.0;
+#if 0
+fprintf (stderr, "ascent: %g descent: %g offsetX: %g, offsetY: %g\n", 
+    ascent(), descent(), offsetX, offsetY);
+#endif
+          d->u.ttf->width (sd, uch, &width);
+          canvas->bitfill(pen.getBackground(), 
+                (int) (m.t0), (int) (m.t1-ascent()), 
+                (int) (width+1.0),
+                (int) (ascent()+descent()+1.0));
         }
-        canvas->fill (pen);
+        /* reverse coordinates Y, negative m.t1 */
+        if (!canvas->beginImage (m.t0+offsetX, m.t1+offsetY, key, pen.getBackground()))
+        {
+          // TODO: add color fonts.
+          canvas->newpath();
+          /* not cached yet */
+          d->u.ttf->draw (canvas, sd, uch, isLRContext);
+          canvas->fill (pen);
+        }
+        canvas->endImage ();
         return true;
     }
     break;
@@ -805,3 +837,17 @@ SFontImpl::clone () const
   CHECK_NEW (ret);
   return ret;
 }
+
+bool
+SFontImpl::needSoftMirror (SS_UCS4 uch, bool isLRContext) const
+{
+  // Sort out strong font types.
+  if (isLR() && !isLRContext) return true;
+  if (isRL() && isLRContext) return true;
+  if (isLR() || isRL()) return false;
+
+  if (getPUARovasType (uch) > 0) return !isLRContext;
+  if (getRovasType (uch) > 0) return isLRContext;
+  // rest is LR, need mirroring in RL context.
+  return !isLRContext;
+}
