@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -20,18 +20,25 @@
 #define SS_MAKKA true
 #include "swindow/SGEngine.h"
 #include "swindow/SImage.h"
-#include "stoolkit/SHashtable.h"
+#include "stoolkit/SBinHashtable.h"
 #include "stoolkit/SProperties.h"
+#include "math.h"
 
 
 static bool cacheOn=true;
 
-typedef SHashtable<SImage> SEImageCache;
+typedef SBinHashtable<SImage*> SEImageCache;
 static SEImageCache imageCache;
 
 static unsigned int cacheSize = 100;
 static unsigned int cacheCount = 0;
 
+#define SS_MITER_ALLOW 2
+#define FILL_REPLAY_LINEWIDTH  0.5 
+#define STROKE_REPLAY_LINEWIDTH 0.25
+
+static void clearImageCache();
+
 /**
  * After this size things won't be cached.
  */
@@ -48,10 +55,10 @@ void
 SGEngine::setCacheOn (bool on)
 {
   cacheOn=on;
-  imageCache.clear();
+  clearImageCache();
 }
 /**
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-04-23
  * This is the abstract widget toolkit package
  */
@@ -135,6 +142,7 @@ SGEngine::setup(double linewidth)
     }
   }
   //colors = scancount * (oversample * oversample) +1;
+  //colors = (oversample * oversample) +1;
   colors = (oversample * oversample) +1;
 }
 
@@ -229,6 +237,7 @@ SGEngine::_closepath()
     linetoInternal (unx, uny);
     unfinished = false;
   }
+  if (pathNow.size () < 2) return;
   double x0 = pathNow[0];
   double y0 = pathNow[1];
   pathNow.append(x0);
@@ -241,34 +250,183 @@ SGEngine::_closepath()
  * TODO: This routine is not yet implemented
  */
 SImage*
-SGEngine::_stroke(int x, int y, unsigned int  width, unsigned int  height)
+SGEngine::_stroke(int _x, int _y, unsigned int  _width, 
+unsigned int  _height, const SPen& pen)
 {
+/*
   if (unfinished)
   {
     linetoInternal (unx, uny);
     unfinished = false;
   }
-  int lines = 0;
-  for (unsigned int a=0; a<pathVector.size(); a++)
-    for (unsigned b=0; b+3<pathVector[a].size(); b=b+2) lines++;
-  if (lines==0) return 0;
+*/
+  if (_height == 0 || _width == 0) {
+    return 0;
+  }
+//fprintf (stderr, "HEIGHT=%d\n", _height);
+  bool offScreen = false;
 #if 0
+  if (unfinished)
+  {
+    linetoInternal (unx, uny);
+    unfinished = false;
+  }
+#endif
+  unsigned int lines = 0;
+  for (unsigned int a=0; a<pathVector.size(); a++)
+    for (unsigned int b=0; b+3<pathVector[a].size(); b=b+2) lines++;
+  if (lines==0) return 0;
+    
+
+  // Mitter can be bad.
+  double cminx =  minx - pen.getLineWidth() * SS_MITER_ALLOW ;
+  if (int (cminx) < _x) 
+  {
+    cminx = double (_x);
+    offScreen = true;
+  }
+  double cminy =  miny - pen.getLineWidth() * SS_MITER_ALLOW ;
+  if (int (cminy) < _y) 
+  {
+    cminy = double (_y);
+    offScreen = true;
+  }
+
+  int origoX=(int) cminx;
+  int origoY=(int) cminy;
+
+  // FIXME : this should be same for fill (x)
+  double cmaxx = maxx+1.0 + pen.getLineWidth() * SS_MITER_ALLOW ;
+  if (cmaxx > double (int(cminx) + int(_width)))
+  {
+    cmaxx = double (int(cminx) + int(_width));
+    offScreen = true;
+  }
+
+  double cmaxy = maxy+1.0 + pen.getLineWidth() * SS_MITER_ALLOW ;
+  if (cmaxy > double (int(cminy) + int (_height)))
+  {
+    cmaxy = double (int(cminy)+ int(_height));
+    offScreen = true;
+  }
+
+  int cwidth = (int) cmaxx - (int) origoX;
+  int cheight = (int) cmaxy - (int) origoY;
+  unsigned int width = (cwidth < 0) ? 1 : cwidth;
+  unsigned int height = (cheight < 0) ? 1 : cheight;
+
+  if (height == 0 || width == 0) return 0;
+  if (cwidth < 0 || cheight < 0)
+  {
+    offScreen = true;
+  }
+
+/*
+  fprintf (stderr, "XXXXX width=%u height=%u origo=%d,%d\n",
+    width, height, origoX, origoY);
+*/
+
+  unsigned int i;
+  unsigned int imageSize = width *height;
+
+  SS_WORD32 *image = new SS_WORD32[imageSize];
+  CHECK_NEW (image);
+  memset (image, 0, imageSize * sizeof (SS_WORD32));
+
   for (unsigned int i=0; i<pathVector.size(); i++)
   {
     for (unsigned int j=0; j+3<pathVector[i].size(); j=j+2)
     {
+      const double* v0 = &pathVector[i].array()[j];
+      
+      strokeScan (image,  width, height, v0, 
+        j+5 < pathVector[i].size(), origoX, origoY, pen, true);
+/*
       double x0 = pathVector[i][j];
       double y0 = pathVector[i][j+1];
       double x1 = pathVector[i][j+2];
       double y1 = pathVector[i][j+3];
-        drawLine (pen, (int) x0, (int) y0, (int) x1, (int) y1);
+*/
     }
+ }
+  for (i=0; i<imageSize; i++)
+  {
+    SS_WORD32 bits = image[i];
+    SS_WORD32 cnt = 0;
+    while (bits)
+    {
+      if (bits&1) cnt++;
+      bits = bits >> 1;
+    }
+    /* count how many bits we have */
+    image[i] = cnt;
   }
-#endif
- pathVector.clear();
- return new SImage(0,1,1);
+ 
+  /**
+   * copy the resulting image
+   */
+  SImage* im = new SImage (image, colors, origoX, 
+        origoY, width, height);
+  CHECK_NEW (im);
+  im->px = (int)newpathX;
+  im->py = (int)newpathY;
+  im->offScreen = offScreen;
+  return im;
 }
 
+void
+SGEngine::strokeScan (SS_WORD32 *image, int width, int height, 
+   const double* vectors, bool join, double origoX, double origoY,
+  const SPen& pen, bool horizontal) 
+  {
+    double dirX = vectors[2] - vectors[0];
+    double dirY = vectors[3] - vectors[1];
+    // 90 degrees
+    double length = sqrt (dirX*dirX + dirY*dirY);
+    if (length < epsylon/100.0) {
+        return;
+    } 
+
+    double penw = pen.getLineWidth() ;
+ //   length, oversample, dirX, dirY);
+    // Unit vector
+/*
+    double dir90UnitX = -dirY / length;
+    double dir90UnitY = dirX / length;
+*/
+
+    // Other direction
+
+    double unitX = dirX / length;
+    double unitY = dirY / length;
+    double inc = 0.5 / oversample;
+    double penr2 = (penw/2) * (penw/2);
+
+    for (double t0 = 0.0; t0<=length; t0=t0+inc) {
+        double x0 = vectors[0] + t0  * unitX; 
+        double y0 = vectors[1] + t0  * unitY; 
+        for (double tx =-penw/2.0; tx<=penw/2.0; tx=tx+inc) {
+            for (double ty =-penw/2.0; ty<=penw/2.0; ty=ty+inc) {
+                double x = x0 + tx;
+                double y = y0 + ty;
+                int indXOver = (int) ((x-(double)origoX) * (double)oversample);
+                int indYOver = (int) ((y-(double)origoY) * (double)oversample);
+                if (tx*tx + ty*ty > penr2) continue;
+                int indX = indXOver/oversample;
+                int indY = indYOver/oversample;
+
+                int bit = ((indXOver % oversample) * oversample) 
+                    + (indYOver % oversample);
+
+                if (indX < 0 || indY < 0) continue;              
+                if (indX >= width || indY >= height) continue;              
+                image[indX + indY * width] |= (1 << bit);
+            }
+        }
+    }
+}
+
+
 SImage*
 SGEngine::_fill(int _x, int _y, unsigned int _width, unsigned int _height)
 {
@@ -355,8 +513,9 @@ SGEngine::_fill(int _x, int _y, unsigned int _width, unsigned int _height)
   for (i=0; i<height; i++)
   {
     delete scanHoriz[i];
+    scanHoriz[i] = 0;
   }
-  delete scanHoriz;
+  delete[] scanHoriz;
 
   /**
    * scan the lines vertically 
@@ -376,8 +535,9 @@ SGEngine::_fill(int _x, int _y, unsigned int _width, unsigned int _height)
     for (i=0; i<width; i++)
     {
       delete scanVert[i];
+      scanVert[i] = 0;
     }
-    delete scanVert;
+    delete[] scanVert;
   }
   for (i=0; i<imageSize; i++)
   {
@@ -391,7 +551,6 @@ SGEngine::_fill(int _x, int _y, unsigned int _width, unsigned int _height)
     /* count how many bits we have */
     image[i] = cnt;
   }
-  pathVector.clear();
   /**
    * copy the resulting image
    */
@@ -698,9 +857,10 @@ SGEngine::scanWinding (SS_InterSection** intersBuff,
       clockwise[i]->remove (j);
     }
     delete clockwise[i];
+    clockwise[i] = 0;
   }
-  delete crossBuff;
-  delete clockwise;
+  delete[] crossBuff;
+  delete[] clockwise;
 }
 
 /**
@@ -808,7 +968,7 @@ SGEngine::scanCrosses (SS_InterSection** intersBuff,
       }
     }
   }
-  delete crossBuff;
+  delete[] crossBuff;
 }
 
 void
@@ -871,21 +1031,75 @@ SGEngine::render (SS_WORD32* image, SS_InterSection** intersBuff,
  * @param y is the y offset
  */
 bool
-SGEngine::newpath (int x, int y, const SString& id)
+SGEngine::beginImage (int x, int y, const SString& id)
 {
   primitive.clear();
+  pathVector.clear();
+  images.clear();
   newpathID = id;
   newpathX = x;
   newpathY = y;
   /* with a bit of a luck we have it in the cache */
-  const SImage* im;
-  if (id.size() && (im = imageCache.get (id)))
+  if (id.size() && imageCache.get (id) != 0)
   {
     return true;
   }
+  return false;
+}
+
+SImage*
+SGEngine::endImage () {
+    primitive.clear();
+    pathVector.clear();
+    SImage* ret = (SImage*) imageCache.get (newpathID);
+    if (ret)
+    {
+        SImage * im = new SImage(*ret);
+        im->setOrigoX (im->getOrigoX() + (int)newpathX - im->px);
+        im->setOrigoY (im->getOrigoY() + (int)newpathY - im->py);
+        return im;
+    }
+    if (images.size() == 0) return 0;
+    // FIXME: merge all images.
+    SImage currentLayer (*images[0]);
+    delete images[0];
+    for (unsigned int i=1; i<images.size(); i++) {
+        currentLayer = currentLayer.addLayer ((*images[i]));
+        delete images[i];
+    } 
+    images.clear();
+    ret = new SImage (currentLayer);
+    if (ret->offScreen)
+    {
+        //fprintf (stderr, "refuse to put off screen image into the cache\n");
+    }
+
+    ret->compress ();
+    if (cacheOn && !ret->offScreen && newpathID.size() > 0) 
+    {
+        cacheCount++;
+        if (cacheCount > cacheSize)
+        {
+            fprintf (stderr, "SGEngine:: clearing cache (%u elements)\n",cacheCount);
+            clearImageCache();
+            cacheCount = 1;
+        }
+// makka
+        imageCache.put (newpathID, new SImage (*ret));
+    //fprintf (stderr, "putting screen image %*.*s into the cache\n",
+     //      SSARGS(newpathID));
+    }
+    return ret;
+}
+
+void
+SGEngine::newpath ()
+{
+  pathVector.clear();
   /* take a first hand look. */
-  SGPrimitive p; p.newpath(); primitive.append (p);
-  return 0;
+  SGPrimitive p;
+  p.newpath();
+  primitive.append (p);
 }
 
 /**
@@ -896,14 +1110,45 @@ SGEngine::newpath (int x, int y, const SString& id)
  * @param height is the desired height
  * @param lw is the lineWidth (if value is less than zero - subpixel *) 
  */
-SImage* 
+void 
 SGEngine::stroke(int x, int y, unsigned int width, unsigned int height, 
-  double lw)
+  const SPen& pen)
 {
-  SGPrimitive p; p.stroke(x, y, width, height);
+  //double lw = pen.getLineWidth();
+  SGPrimitive p; 
+  p.stroke(x, y, width, height);
   primitive.append (p);
-  _replay(lw);
-  return _stroke (x, y, width, height);
+  _replay(STROKE_REPLAY_LINEWIDTH);
+  if (unfinished)
+  {
+    linetoInternal (unx, uny);
+    unfinished = false;
+  }
+  if (pathNow.size()) pathVector.append (pathNow);
+  pathNow.clear();
+
+  SImage* ret = _stroke (x, y, width, height, pen);
+  if (ret) {
+    SImage colsi = ret->colorize(pen.getForeground());
+    delete ret;
+    ret = new SImage(colsi);
+    images.append (ret);
+  }
+}
+
+void
+SGEngine::fill (int x, int y, unsigned int width, unsigned int height, 
+  const SPen& pen)
+{
+  double lw = pen.getLineWidth();
+  SImage* si = fillInternal (x, y, width, height, lw);
+  //fprintf  (stderr, "Image Layer appended %lx\n", (unsigned long) ret);
+  if (si) {
+    SImage colsi = si->colorize(pen.getForeground());
+    delete si;
+    si = new SImage(colsi);
+    images.append (si);
+  }
 }
 
 /**
@@ -916,35 +1161,23 @@ SGEngine::stroke(int x, int y, unsigned int width, unsigned int height,
  * newpathID is not "" and the image is not off the screen.
  */
 SImage* 
-SGEngine::fill (int x, int y, unsigned int width, unsigned int height, 
+SGEngine::fillInternal (int x, int y, unsigned int width, unsigned int height, 
   double lw)
 {
   SGPrimitive p; p.fill (x, y, width, height); primitive.append (p);
 
-  if (!cacheOn || newpathID.size()==0)
+  if (newpathID.size()==0)
   {
     //fprintf (stderr, "screen image %*.*s cacheOn=%d\n",
      //      SSARGS(newpathID), (int)cacheOn);
-    _replay(lw);
-    return _fill (x, y, width, height);
+    _replay(FILL_REPLAY_LINEWIDTH);
+    SImage* ret = _fill (x, y, width, height);
+    pathVector.clear();
+    return ret;
   }
 
-  if (imageCache.get (newpathID))
-  {
-    primitive.clear();
-    SImage * im = new SImage(imageCache[newpathID]);
-
-    im->setOrigoX (im->getOrigoX() + (int)newpathX - im->px);
-    im->setOrigoY (im->getOrigoY() + (int)newpathY - im->py);
-
-    //im->px = im->getOrigoX() + (int)newpathX - im->px;
-    //im->py = im->getOrigoY() + (int)newpathY - im->py;
-    //fprintf (stderr, "screen image %*.*s is already in the cache\n",
-     //      SSARGS(newpathID));
-    return im;
-  }
   /*--------- no luck with cached image. -------------*/    
-  _replay(lw);
+  _replay(FILL_REPLAY_LINEWIDTH);
   SImage *ii = _fill (x, y, width, height);
   if (ii==0) return 0;
 
@@ -953,25 +1186,7 @@ SGEngine::fill (int x, int y, unsigned int width, unsigned int height,
 
 //fprintf (stderr, "created one with newpathX=%d, newpathY=%d\n",
  //  (int)newpathX, (int)newpathY);
-
-  if (ii->offScreen)
-  {
-    //fprintf (stderr, "refuse to put off screen image into the cache\n");
-  }
-  else
-  {
-    ii->compress ();
-    cacheCount++;
-    if (cacheCount > cacheSize)
-    {
-      fprintf (stderr, "SGEngine:: clearing cache (%u elements)\n",cacheCount);
-      imageCache.clear ();
-      cacheCount = 1;
-    }
-    imageCache.put (newpathID, *ii);
-    //fprintf (stderr, "putting screen image %*.*s into the cache\n",
-     //      SSARGS(newpathID));
-  }
+  pathVector.clear();
   return ii;
 }
 
@@ -1023,7 +1238,6 @@ SGEngine::_replay (double lineWidth)
       break;
     }
   }
-  primitive.clear();
 }
 
 
@@ -1104,3 +1318,17 @@ SGEngine::getCurrentMatrix() const
 {
   return SS_Matrix2D(matrix[matrix.size()-1]);
 }
+
+static void clearImageCache() {
+    for (unsigned int i=0; i<imageCache.size(); i++)
+    {
+        for (unsigned int j=0; j<imageCache.size(i); j++)
+        {
+            SImage* e = imageCache.get (i, j);
+            if (e != 0) delete e;
+        }
+    }
+    imageCache.clear ();
+    cacheCount = 0;
+}
+
