@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -25,14 +25,16 @@
  * Bitmap font support
  */
 #ifndef USE_WINAPI 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include "swindow/SAwt.h"
-#include "swindow/sx11/SX11Impl.h"
+# if USE_X11
+#  include <X11/Xlib.h>
+#  include <X11/Xutil.h>
+#  include "swindow/SAwt.h"
+#  include "swindow/sx11/SX11Impl.h"
+# endif
 #endif
 
 /**
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-04-23
  * This is a postscript renderer for yudit.
  */
@@ -48,13 +50,89 @@ SPostscript::SPostscript (const SWriter& wr, SMedia m, SOrientation o)  : out (w
   unsigned int h;
   switch (m)
   {
+  case A3:
+    w = 842;
+    h = 1190;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
   case A4:
+    w = 595;
+    h = 842;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
+  case A5:
+    w = 420;
+    h = 595;
+    widthMargin = 45;
+    heightMargin = 34;
+    break;
+  case B4:
+    w = 792;
+    h = 1032;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
+  case B5:
+    w = 516;
+    h = 729;
+    widthMargin = 45;
+    heightMargin = 34;
+    break;
+  case Executive:
+    w = 540;
+    h = 720;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
+  case Folio:
+    w = 612;
+    h = 936;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
+  case Ledger:
+    w = 980;
+    h = 792;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
+  case Legal:
+    w = 612;
+    h = 1008;
+    widthMargin = 64;
+    heightMargin = 48;
+    break;
+  case Letter:
+    w = 612;
+    h = 792;
+    widthMargin = 24;
+    heightMargin = 34;
+    break;
+  case Quarto:
+    w = 610;
+    h = 780;
+    widthMargin = 24;
+    heightMargin = 48;
+    break;
+  case Statement:
+    w = 396;
+    h = 612;
+    widthMargin = 24;
+    heightMargin = 48;
+    break;
+  case Tabloid:
+    w = 792;
+    h = 1028;
+    widthMargin = 24;
+    heightMargin = 48;
+    break;
   default:
     w = 595;
     h = 842;
     widthMargin = 64;
     heightMargin = 48;
-    //heightMargin = 64;
   }
   if (o==PORTRAIT)
   {
@@ -132,12 +210,30 @@ SPostscript::open (bool background)
   SStringVector head("%!PS-Adobe-3.0");
   SString bb("%%BoundingBox: ");
 
-  bb.print (widthMargin); bb.append (" ");
-  bb.print (heightMargin); bb.append (" ");
-  bb.print (width - 2 * widthMargin); bb.append (" ");
-  bb.print (height - 2 * heightMargin); bb.append (" ");
-  
-  head.append ("%%BoundingBox: 64 48 531 794");
+  switch (orientation)
+  {
+  case LANDSCAPE:
+ 
+        bb.print (heightMargin); bb.append (" ");
+        bb.print (widthMargin); bb.append (" ");
+        bb.print (height - heightMargin); bb.append (" ");
+        bb.print (width - widthMargin);
+        head.append (bb);
+ 
+    break;
+ 
+  case PORTRAIT:
+  default:
+ 
+        bb.print (widthMargin); bb.append (" ");
+        bb.print (heightMargin); bb.append (" ");
+        bb.print (width - widthMargin); bb.append (" ");
+        bb.print (height - heightMargin);
+        head.append (bb);
+ 
+    break;
+  }
+
   head.append ("%%Title: Untitled");
   SString creator ("%%Creator: yudit "); creator.append (SD_YUDIT_VERSION);
   creator.append (" GNU (C) Gaspar Sinai");
@@ -165,12 +261,37 @@ SPostscript::open (bool background)
   SString m ("%%DocumentMedia: ");
   switch (media)
   {
+  case Legal:
+     m.append ("Legal "); break;
+  case Letter:
+     m.append ("Letter "); break;
+  case Tabloid:
+     m.append ("Tabloid "); break;
+  case Ledger:
+     m.append ("Ledger "); break;
+  case B4:
+     m.append ("B4 "); break;
+  case A3:
+     m.append ("A3 "); break;
   case A4:
+     m.append ("A4 "); break;
+  case A5:
+     m.append ("A5 "); break;
+  case B5:
+     m.append ("B5 "); break;
   default:
      m.append ("A4 ");
   }
-  m.print (width); m.append (" ");
-  m.print (height); m.append (" 0 () ()");
+  if (orientation == LANDSCAPE)
+  {
+    m.print (height); m.append (" ");
+    m.print (width); m.append (" 0 () ()");
+  }
+  else 
+  {
+    m.print (width); m.append (" ");
+    m.print (height); m.append (" 0 () ()");
+  }
   head.append (m);
   // windows gs goes crazy if I uncomment this.
   //head.append ("");
@@ -236,7 +357,25 @@ SPostscript::newPage()
     }
     currentPage++;
     char buff[128];
-    sprintf (buff, "\n%%%%Page: %u %u\nsave\n", pages, currentPage);
+ 
+  switch (orientation)
+  {
+  case LANDSCAPE:
+ 
+          sprintf (buff, "\n%%%%Page: %u %u\nsave\n90 rotate\n0 -%d.000 translate\n", pages, currentPage, height);
+ 
+    break;
+ 
+  case PORTRAIT:
+  default:
+ 
+          sprintf (buff, "\n%%%%Page: %u %u\nsave\n", pages, currentPage);
+ 
+    break;
+  }
+ 
+ 
+ 
     writeString (buff);
   }
 }
@@ -252,7 +391,7 @@ SPostscript::newPage()
  * @param id is the id to print.
  */
 bool
-SPostscript::_newpath (double _x, double _y, const SString& _id)
+SPostscript::_beginImage (double _x, double _y, const SString& _id)
 {
   if (!cacheIDMap.get (_id)) return false;
   char ins[128];
@@ -337,7 +476,7 @@ SPostscript::cacheOn (bool on)
  * @return true if no more draing is needed.
  */
 bool
-SPostscript::newpath (double _x, double _y, const SString& _id)
+SPostscript::beginImage (double _x, double _y, const SString& _id, const SColor& background)
 {
   /* is it in the background ? */
   if (timer)
@@ -353,28 +492,38 @@ SPostscript::newpath (double _x, double _y, const SString& _id)
   {
     /* This is a hack. now I requere you to cahce first. */
     /* for some reason this one was inserting a newpath with no fill */
-    if (!_newpath (_x, _y, _id))
-    {
+    // FIXME
+    _beginImage (_x, _y, _id);
+    return true;
+  }
+  /* have to draw it anyway later. */
+  if (_id.size() == 0) return true;
+  return false;
+}
+
+void
+SPostscript::newpath () 
+{
+  /* put it in the cache */
+  if (!isCacheOn) 
+  {
 //fprintf (stderr, "failed %*.*s\n", SSARGS(_id));
+// FIXME
+#if 0
       char arr[64];
       int ix =  (int) (1000.0 * _x);
       int iy =  (int) (1000.0 * ((double)height-_y));
       sprintf (arr, "gsave %s%d.%03d %s%d.%03d translate newpath\n", 
         SG(ix), ix/1000, SM(ix), SG(iy), iy/1000, SM(iy));
+      // BUG-2011-08-04 Don't write. This happens between U+0000...U+001F
       //writeString (arr);
-      return true;
-    }
-    return true;
+#endif
+      return ;
   }
-  /* have to draw it anyway later. */
-  if (_id.size() == 0) return true;
-
-  /* put it in the cache */
   cacheCurrent.append ("newpath\n");
-
-  return false;
 }
 
+
 /**
  * have to call fill if newpath does fail. 
  * This routine should not be called if cache is off.
@@ -386,6 +535,21 @@ SPostscript::fill (const SPen& pen)
   fill();
 }
 
+void
+SPostscript::endImage () 
+{
+  if (isCacheOn)
+  {
+    if (cache.get (cacheID)) return;
+    cache.put (cacheID, cacheCurrent);
+    char buff[64];
+    sprintf (buff, "Glyph%u", cacheIDMapCount++);
+    cacheIDMap.put (cacheID, SString(buff));
+    cacheCurrent.clear();
+    cacheID.clear();
+  }
+}
+
 /**
  * have to call fill if newpath does fail. 
  * This routine should not be called if cache is off.
@@ -398,16 +562,12 @@ SPostscript::fill ()
   {
     if (cache.get (cacheID)) return;
     cacheCurrent.append ("fill\n");
-    cache.put (cacheID, cacheCurrent);
-    char buff[64];
-    sprintf (buff, "Glyph%u", cacheIDMapCount++);
-    cacheIDMap.put (cacheID, SString(buff));
-    cacheCurrent.clear();
-    cacheID.clear();
   }
   else 
   {
-    if (cache.get (cacheID) == 0) writeString ("fill grestore\n");
+    // BUG-2011-08-04 Don't write. This happens between U+0000...U+001F
+    // Originally only newpath was commented out.
+    // if (cache.get (cacheID) == 0) writeString ("fill grestore\n");
   }
 }
 
@@ -421,12 +581,8 @@ SPostscript::stroke (const SPen& pen)
 {
   if (isCacheOn)
   {
+    if (cache.get (cacheID)) return;
     cacheCurrent.append ("stroke\n");
-    char buff[64];
-    sprintf (buff, "Glyph%u", cacheIDMapCount++);
-    cacheIDMap.put (cacheID, SString(buff));
-    cacheCurrent.clear();
-    cacheID.clear();
   }
   else 
   {
@@ -589,12 +745,12 @@ SPostscript::bitfont (const SPen& pen, double x, double y,
   SString key("NF");
   key.append (SString((long)native));
   key.append (SString(data, len));
-  if (newpath (x, y, key))
+  if (beginImage (x, y, key, pen.getBackground()))
   {
-    fill (pen);
+    endImage();
     return;
   }
-
+  newpath();
   /* FIXME: draw font here */
   int bits = DefaultDepth (impl->display, impl->screen); 
   int nchars = len/2;
@@ -620,6 +776,7 @@ SPostscript::bitfont (const SPen& pen, double x, double y,
   if (pixmap==None)
   {
     fill (pen);
+    endImage();
     fprintf (stderr, "(Pixmap) no luck in prining bitmap font.\n");
     return;
   }
@@ -640,6 +797,7 @@ SPostscript::bitfont (const SPen& pen, double x, double y,
   {
     fprintf (stderr, "(Image) no luck in prining bitmap font.\n");
     fill (pen);
+    endImage();
     return;
   }
 
@@ -687,6 +845,7 @@ SPostscript::bitfont (const SPen& pen, double x, double y,
   fill (pen);
 # endif
 #endif
+  endImage();
 }
 bool
 SPostscript::hasNative () const
@@ -731,9 +890,9 @@ SPostscript::bitpoint (const SColor& fg, int x, int y)
   SString key("BP");
   key.append (SString((long)x));
   key.append (SString((long)y));
-  if (newpath (x, y, key))
+  if (beginImage (x, y, key, SColor ("white")))
   {
-    fill ();
+    endImage ();
     return;
   }
   isUgly = true;
@@ -765,10 +924,47 @@ void
 SPostscript::bitpoints (const SColor& fg, const int* x, const int* y, 
          unsigned int _size)
 {
-  for (unsigned int i=0; i<_size; i++)
+  if (_size<=0) return;
+  SString key("BPS");
+  unsigned int i;
+  for (i=0; i<_size; i++)
   {
-    bitpoint (fg, x[i], y[i]);
+    key.append (SString(((char*) &x[i]), sizeof(int)));
+    key.append (SString(((char*) &y[i]), sizeof(int)));
   }
+  if (beginImage (x[0], y[0], key, SColor ("white")))
+  {
+    endImage ();
+    return;
+  }
+  newpath();
+  for (i=0; i<_size; i++)
+  {
+    double squareX[4];
+    double squareY[4];
+    double delta = 1.0;
+
+    squareX[0] = (double)x[i];
+    squareY[0] = (double)y[i];
+
+    squareX[1] = squareX[0] + delta;
+    squareY[1] = squareY[0];
+
+    squareX[2] = squareX[0] + delta;
+    squareY[2] = squareY[0] + delta;
+
+    squareX[3] = squareX[0];
+    squareY[3] = squareY[0] + delta;
+
+    moveto (squareX[0], squareY[0]);
+    lineto (squareX[1], squareY[1]);
+    lineto (squareX[2], squareY[2]);
+    lineto (squareX[3], squareY[3]);
+    closepath();
+  }
+  fill ();
+  endImage ();
+  isUgly = true;
 }
 
 bool
