@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -78,58 +78,69 @@ SX11Window::setPixmapCacheOn (bool on)
 
 SX11Window::SX11Window(const SString& n, SX11Impl* _impl, long _id) : name (n),
  inputMethod (_impl, _id, SD_YUDIT_EVENT_MASK),
- pen (SColor(0), SColor(0xffffffff)),
- background (_impl, SS_WORD32(0xffffffff))
+ background (SX11Color(_impl, SS_WORD32(0xffffffff))),
+ pen (SColor(0), SColor(0xffffffff))
 {
+  impl = _impl;
+  dbindex = 0;
+
   isChild = 0;
   mapNotified = false;
-  id = _id;
-  impl = _impl;
-  impl->addWindow (id, this);
+  exposedOnce = false;
+  dbuffer[0].id = _id;
+  dbuffer[1].id = None;
+
+  dbuffer[0].gcFD = 0;
+
+  dbuffer[0].gcFG = (unsigned long) pen.getForeground().getValue();
+  impl->addWindow (dbuffer[0].id, this);
   XGCValues   gcv;
   gcv.foreground = SX11Color(impl, pen.getForeground()).getPixelValue();
   gcv.background = background.getPixelValue();
   clipped = false;
-  gcFD = 0;
-  gcFG = (unsigned long) pen.getForeground().getValue();
 
-  gc = XCreateGC (impl->display, (Window)id, GCForeground
-    | GCBackground, &gcv); 
-  gc_default = XCreateGC (impl->display, (Window)id, 0, &gcv);
+  dbuffer[0].gc = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground, &gcv); 
+
+  dbuffer[0].gc_default = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+      0, &gcv);
+
   gcv.function = GXand;
-  gc_draw_and = XCreateGC (impl->display, (Window)id, GCForeground
-    | GCBackground | GCFunction, &gcv); 
+  dbuffer[0].gc_draw_and = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground | GCFunction, &gcv); 
+
   gcv.function = GXor;
-  gc_draw_or = XCreateGC (impl->display, (Window)id, GCForeground
-    | GCBackground | GCFunction, &gcv); 
+  dbuffer[0].gc_draw_or = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+     GCForeground | GCBackground | GCFunction, &gcv); 
 
   setBackground (background);
   engine = 0;
   shown = false;
-  XSelectInput (impl->display, (Window) id, SD_YUDIT_EVENT_MASK);
+  XSelectInput (impl->display, (Window) dbuffer[0].id, SD_YUDIT_EVENT_MASK);
 }
 
 SX11Window::~SX11Window()
 {
-  XFreeGC (impl->display, gc);
-  XFreeGC (impl->display, gc_default);
-  XFreeGC (impl->display, gc_draw_and);
-  XFreeGC (impl->display, gc_draw_or);
-  impl->deleteWindow (id, this);
+  freeDoubleBuffer ();
+  XFreeGC (impl->display, dbuffer[0].gc);
+  XFreeGC (impl->display, dbuffer[0].gc_default);
+  XFreeGC (impl->display, dbuffer[0].gc_draw_and);
+  XFreeGC (impl->display, dbuffer[0].gc_draw_or);
+  impl->deleteWindow (dbuffer[0].id, this);
   if (engine) delete engine;
 }
 
 void
 SX11Window::show ()
 {
-  impl->show (id);
+  impl->show (dbuffer[0].id);
   shown = true;
 }
 
 void
 SX11Window::hide ()
 {
-  impl->hide (id);
+  impl->hide (dbuffer[0].id);
   shown = false;
 }
 
@@ -137,19 +148,41 @@ SX11Window::hide ()
  * Drawing routines inherited from SCanvas
  */
 bool
-SX11Window::newpath (double _x, double _y, const SString& _id)
+SX11Window::beginImage (double _x, double _y, const SString& _id, const SColor& background)
 {
+  pen = SPen(pen.getForeground(), background, pen.getLineWidth());
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::newpath called without beginDoubleBuffer\n");
+    /* this will always return false */
+    return engine->beginImage ((int)_x, (int)_y, "");
+  }
   if (engine ==0) engine = new SGEngine();
   if (isCacheOn)
   {
-    return engine->newpath ((int)_x, (int)_y, _id);
+    return engine->beginImage ((int)_x, (int)_y, _id);
   }
   else
   {
     /* this will always return false */
-    return engine->newpath ((int)_x, (int)_y, "");
+    return engine->beginImage ((int)_x, (int)_y, "");
   }
 }
+void
+SX11Window::newpath ()
+{
+  if (engine==0) return;
+    return engine->newpath ();
+}
+void
+SX11Window::fill (const SPen& _pen) {
+  if (engine==0) return;
+  if (pen != _pen)
+  {
+     pen = _pen;
+  }
+  engine->fill (0, 0, getWidth(), getHeight(), pen);
+}
 
 #define SS_SHADING_COLORS (SD_OVERSAMPLE * SD_OVERSAMPLE +1)
 #define SS_DOUBLE_SCAN 1
@@ -158,16 +191,15 @@ SX11Window::newpath (double _x, double _y, const SString& _id)
  * fill and return the resulting image for better cahcing.
  */
 void
-SX11Window::fill (const SPen& _pen)
+SX11Window::endImage ()
 {
-  if (pen != _pen)
+  if (dbuffer[dbindex].id == None)
   {
-     pen = _pen;
+    fprintf (stderr, "SX11Window::fill called without beginDoubleBuffer\n");
+    return;
   }
-
   if (engine==0) return;
-
-  SImage* si= engine->fill (0, 0, getWidth(), getHeight(), pen.getLineWidth());
+  SImage* si= engine->endImage ();
   if (si==0) return; /* offscreen */
 
   /* Use the putimage that does some cahcheing. */
@@ -230,8 +262,9 @@ SX11Window::fill (const SPen& _pen)
       XPutPixel (ximage, x, y, rpvle); 
     }
   }
-  XPutImage (impl->display, (Window) id, gc_draw_and, ximage, 0, 0,
-     si->getOrigoX(), si->getOrigoY(), 
+  XPutImage (impl->display, (Drawable) dbuffer[dbindex].id, 
+     dbuffer[dbindex].gc_draw_and, 
+     ximage, 0, 0, si->getOrigoX(), si->getOrigoY(), 
      imageWidth, imageHeight);
 
   /* Then blot the image */
@@ -239,13 +272,22 @@ SX11Window::fill (const SPen& _pen)
   {
     for (int x=0; x<imageWidth; x++)
     {
-      SS_WORD32 sh = si->getShade (x, y);
-      SS_WORD32  rpvle = (sh==0) ? 0 : colors[si->getShade (x, y)]->getPixelValue();
-      XPutPixel (ximage, x, y, rpvle); 
+      if (shades > 0) {
+        SS_WORD32 sh = si->getShade (x, y);
+        SS_WORD32  rpvle = (sh==0) ? 0 : colors[si->getShade (x, y)]->getPixelValue();
+        XPutPixel (ximage, x, y, rpvle); 
+      } else {
+        SColor scolor(si->getShade (x, y));
+        SColor bg (pen.getBackground());
+        bg.blend (scolor);
+        SS_WORD32  rpvle = SX11Color(impl, bg).getPixelValue();
+        XPutPixel (ximage, x, y, rpvle); 
+      }
     }
   }
 
-  XPutImage (impl->display, (Window) id, gc_draw_or, ximage, 0, 0,
+  XPutImage (impl->display, (Drawable) dbuffer[dbindex].id, 
+     dbuffer[dbindex].gc_draw_or, ximage, 0, 0,
      si->getOrigoX(), si->getOrigoY(), 
      imageWidth, imageHeight);
 
@@ -270,9 +312,7 @@ SX11Window::stroke (const SPen& _pen)
   {
      pen = _pen;
   }
-  SImage *i = engine->stroke(0,0, getWidth(),  getHeight(), _pen.getLineWidth());
-  if (i==0) return;
-  delete i;
+  engine->stroke(0,0, getWidth(),  getHeight(), pen);
 }
 
 /**
@@ -326,6 +366,11 @@ SX11Window::closepath()
 void 
 SX11Window::putImage (int _x, int _y, const SImage& im)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::putImage called without beginDoubleBuffer\n");
+    return;
+  }
   if (cacheCount > cacheSize)
   {
       fprintf (stderr, "SX11Window:: clearing cache (%u elements)\n",
@@ -378,6 +423,8 @@ SX11Window::putImage (int _x, int _y, const SImage& im)
   bool createMask = (mask==0);
   bool createPixmap = (pixmap==0);
 
+//fprintf (stderr, "createPixmap=%d %*.*s\n", (int)createPixmap, SSARGS(key));
+
   /* Pixmap is always added, mask is on-demand */
   if (createPixmap)
   {
@@ -403,9 +450,11 @@ fprintf (stderr, "\n");
   {
 //fprintf (stderr, "Using cache X11\n");
   }
-  XCopyArea(impl->display, mask, (Window)id, gc_draw_and, 0, 0, 
+  XCopyArea(impl->display, mask, (Drawable)dbuffer[dbindex].id, 
+     dbuffer[dbindex].gc_draw_and, 0, 0, 
      im.getWidth(), im.getHeight(), _x, _y);
-  XCopyArea(impl->display, pixmap, (Window)id, gc_draw_or, 0, 0, 
+  XCopyArea(impl->display, pixmap, (Drawable)dbuffer[dbindex].id, 
+    dbuffer[dbindex].gc_draw_or, 0, 0, 
     im.getWidth(), im.getHeight(), _x, _y);
 
   /* This should have the foreground and background in it */
@@ -415,6 +464,8 @@ fprintf (stderr, "\n");
     if (createPixmap)
     {
       imageCache.put (key, pixmap);
+//Pixmap pixmap2 = imageCache.get (key);
+//fprintf (stderr, "addPixmap=%lx %*.*s\n", (unsigned long)pixmap2, SSARGS(key));
       //cacheCount += im.getWidth() * im.getHeight() / SS_IMAGE_SIZE;
       cacheCount++;
       //fprintf (stderr, "cacheCount=%u cacheSize=%u\n", cacheCount, cacheSize);
@@ -449,6 +500,11 @@ fprintf (stderr, "\n");
 void
 SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::createColoredPixmap called without beginDoubleBuffer\n");
+    return;
+  }
   int imageWidth = (int) im.getWidth();
   int imageHeight = (int) im.getHeight();
 
@@ -472,9 +528,11 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
       }
     }
 
-    Pixmap xshape = XCreatePixmap (impl->display, (Window) id, 
+    Pixmap xshape = XCreatePixmap (impl->display, 
+      (Drawable) dbuffer[dbindex].id, 
       im.getWidth(), im.getHeight(), bits);
-    XPutImage (impl->display, xshape, gc_default, ximage, 0, 0, 0, 0, 
+    XPutImage (impl->display, xshape, dbuffer[dbindex].gc_default, 
+       ximage, 0, 0, 0, 0, 
       im.getWidth(), im.getHeight());
     *m = xshape;
   }
@@ -486,16 +544,20 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
     {
       SColor c (im.getShade (x, y));
       SColor bg (pen.getBackground());
-      if (c.alpha != 0) bg.blend (c);
-      SS_WORD32  rpvle = SX11Color(impl, bg).getPixelValue();
+      //if (c.alpha != 0) bg.blend (c);
+      bg.blend (c);
+      // CHANGED 2020-05-24
+      SS_WORD32  rpvle = (c.alpha == 0) 
+        ? 0 : SX11Color(impl, bg).getPixelValue();
       XPutPixel (ximage, x, y, rpvle); 
     }
   }
 
-  Pixmap xpixmap = XCreatePixmap (impl->display, (Window) id, 
+  Pixmap xpixmap = XCreatePixmap (impl->display, (Drawable)dbuffer[dbindex].id, 
      im.getWidth(), im.getHeight(), bits);
 
-  XPutImage (impl->display, xpixmap, gc_default, ximage, 0, 0, 0, 0, 
+  XPutImage (impl->display, xpixmap, dbuffer[dbindex].gc_default, 
+      ximage, 0, 0, 0, 0, 
      im.getWidth(), im.getHeight());
 
   XDestroyImage (ximage);
@@ -511,6 +573,11 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 void
 SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::createShadedPixmap called without beginDoubleBuffer\n");
+    return;
+  }
   int imageWidth = (int) im.getWidth();
   int imageHeight = (int) im.getHeight();
 
@@ -535,10 +602,12 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
         XPutPixel (ximage, x, y, rpvle); 
       }
     }
-    Pixmap xshape = XCreatePixmap (impl->display, (Window) id, 
+    Pixmap xshape = XCreatePixmap (impl->display, 
+      (Drawable) dbuffer[dbindex].id, 
       im.getWidth(), im.getHeight(), bits);
 
-    XPutImage (impl->display, xshape, gc_default, ximage, 0, 0, 0, 0, 
+    XPutImage (impl->display, xshape, dbuffer[dbindex].gc_default, 
+      ximage, 0, 0, 0, 0, 
       im.getWidth(), im.getHeight());
     *m = xshape;
   }
@@ -553,6 +622,7 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
   for (i=0; i<shades; i++)
   {
       SColor bg (pen.getBackground());
+      //SGC SColor bg (SColor("#808080"));
       SColor fg (pen.getForeground().red,
         pen.getForeground().green, 
         pen.getForeground().blue,
@@ -574,10 +644,12 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
     }
   }
 
-  Pixmap xpixmap = XCreatePixmap (impl->display, (Window) id, 
+  Pixmap xpixmap = XCreatePixmap (impl->display, 
+     (Drawable) dbuffer[dbindex].id, 
      im.getWidth(), im.getHeight(), bits);
 
-  XPutImage (impl->display, xpixmap, gc_default, ximage, 0, 0, 0, 0, 
+  XPutImage (impl->display, xpixmap, dbuffer[dbindex].gc_default, 
+     ximage, 0, 0, 0, 0, 
      im.getWidth(), im.getHeight());
 
   for (i=0; i<shades; i++)
@@ -598,13 +670,19 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 void
 SX11Window::createMaskedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::createMaskedPixmap called without beginDoubleBuffer\n");
+    return;
+  }
   int imageWidth = (int) im.getWidth();
   int imageHeight = (int) im.getHeight();
 
   /* First wipe out the shape */
   if (m != 0)
   {
-    Pixmap xshape = XCreatePixmap (impl->display, (Window) id, 
+    Pixmap xshape = XCreatePixmap (impl->display, 
+      (Drawable) dbuffer[dbindex].id, 
        im.getWidth(), im.getHeight(), 1);
     XGCValues   gcv;
     gcv.background = 0x0;
@@ -650,10 +728,12 @@ SX11Window::createMaskedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
     }
   }
 
-  Pixmap xpixmap = XCreatePixmap (impl->display, (Window) id, 
+  Pixmap xpixmap = XCreatePixmap (impl->display, 
+     (Drawable) dbuffer[dbindex].id, 
      im.getWidth(), im.getHeight(), bits);
 
-  XPutImage (impl->display, xpixmap, gc_default, ximage, 0, 0, 0, 0, 
+  XPutImage (impl->display, xpixmap, dbuffer[dbindex].gc_default, 
+     ximage, 0, 0, 0, 0, 
      im.getWidth(), im.getHeight());
 
   XDestroyImage (ximage);
@@ -665,7 +745,8 @@ SX11Window::setBackground(const SColor &color)
 {
   SX11Color c(impl, color);
   background = c;
-  XSetWindowBackground (impl->display, (Window)id, background.getPixelValue());
+  XSetWindowBackground (impl->display, (Window)dbuffer[0].id, 
+     background.getPixelValue());
   pen = SPen(pen.getForeground(), color, pen.getLineWidth());
 }
 
@@ -711,12 +792,13 @@ SX11Window::popmatrix()
 void
 SX11Window::resize (unsigned int _width, unsigned int _height)
 {
-//  fprintf (stderr, "resizing from: %u %u to %u %u\n",
-// width, height, _width, _height);
+  //fprintf (stderr, "resizing from: %lx   %u %u to %u %u\n",
+  //(unsigned long) this, xwidth, yheight, _width, _height);
+
   if (getWidth() == _width && getHeight() == _height)return;
   if (_width == 0) _width = 1;
   if (_height == 0) _height = 1;
-  XResizeWindow (impl->display, (Window) id, _width, _height);
+  XResizeWindow (impl->display, (Window) dbuffer[0].id, _width, _height);
   setSize(_width, _height);
 }
 
@@ -724,7 +806,7 @@ void
 SX11Window::move (int _x, int _y)
 {
   if (getPositionX() == _x && getPositionY() == _y)return;
-  XMoveWindow (impl->display, (Window) id, _x, _y);
+  XMoveWindow (impl->display, (Window) dbuffer[0].id, _x, _y);
   setPosition(_x, _y);
 }
 /**
@@ -739,31 +821,39 @@ void
 SX11Window::bitfont (const SPen& _pen, double x, double y, 
   void* native, char* data, unsigned int len)
 {
-  if (gcFD != (unsigned long) native)
+  if (dbuffer[dbindex].id == None)
   {
-    XSetFont (impl->display, gc, (Font) native);
-    gcFD  = (unsigned long) native;
+    fprintf (stderr, "SX11Window::bitfont called without beginDoubleBuffer\n");
+    return;
   }
-  if (gcFG != (unsigned long) _pen.getForeground().getValue())
+  if (dbuffer[dbindex].gcFD != (unsigned long) native)
   {
-    XSetForeground (impl->display, gc,
+    XSetFont (impl->display, dbuffer[dbindex].gc, (Font) native);
+    dbuffer[dbindex].gcFD  = (unsigned long) native;
+  }
+  if (dbuffer[dbindex].gcFG != (unsigned long) _pen.getForeground().getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc,
        SX11Color(impl, _pen.getForeground()).getPixelValue());
-    gcFG = (unsigned long) _pen.getForeground().getValue();
+    dbuffer[dbindex].gcFG = (unsigned long) _pen.getForeground().getValue();
   }
-  XDrawString16 (impl->display, (Window) id, gc, (int)x, (int)y, 
+  XDrawString16 (impl->display, (Drawable) dbuffer[dbindex].id, 
+    dbuffer[dbindex].gc, (int)x, (int)y, 
     (XChar2b*) data, len/2);
 }
 
 void
 SX11Window::setParent (SWindow* w, int x, int y)
 {
+  //fprintf (stderr, "reparent %lx\n", (unsigned long) this);
   SX11Window* x11w = (SX11Window*) w;
-  XReparentWindow (impl->display, (Window) id, (Window) x11w->id, x, y);
+  XReparentWindow (impl->display, (Window) dbuffer[0].id, (Window) 
+      x11w->dbuffer[0].id, x, y);
   isChild  = true;
 }
 
 /**
- * Clear a reagion (set it to the background)
+ * Clear a region (set it to the background)
  * @param x is the upper left corner
  * @param y is the upper top corner
  * @param width is the width of the region to clear
@@ -772,9 +862,15 @@ SX11Window::setParent (SWindow* w, int x, int y)
 void
 SX11Window::clear (int x, int y, unsigned int width, unsigned int height)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::clear called without beginDoubleBuffer\n");
+    return;
+  }
   bitfill (background, x, y, width, height);
 /* This one does not takes clip */
 /*
+  // does not work with pixmaps
   XClearArea (impl->display, (Window) id, x, y, width, height, 0);
 */
 }
@@ -792,9 +888,15 @@ void
 SX11Window::copy (int x, int y, unsigned int width, unsigned int height, 
   int tox, int toy)
 {
-  XCopyArea (impl->display, (Window) id, (Window) id, 
-      gc, x, y, width, height, tox, toy);
-  impl->moveRedrawEvent (id, tox-x, toy-y);
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::copy called without beginDoubleBuffer\n");
+    return;
+  }
+  XCopyArea (impl->display, (Drawable) dbuffer[dbindex].id, 
+     (Drawable) dbuffer[dbindex].id, 
+      dbuffer[dbindex].gc, x, y, width, height, tox, toy);
+  impl->moveRedrawEvent (dbuffer[0].id, tox-x, toy-y);
 }
 
 /**
@@ -808,12 +910,19 @@ void
 SX11Window::bitfill (const SColor& bg, int x, int y, 
  unsigned int width, unsigned int height)
 {
-  if (gcFG != (unsigned long)  bg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, bg).getPixelValue());
-    gcFG = (unsigned long)  bg.getValue();
+    fprintf (stderr, "SX11Window::bitfill called without beginDoubleBuffer\n");
+    return;
   }
-  XFillRectangle (impl->display, (Window) id, gc, x, y, width, height);
+  if (dbuffer[dbindex].gcFG != (unsigned long)  bg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, 
+       SX11Color(impl, bg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  bg.getValue();
+  }
+  XFillRectangle (impl->display, (Drawable) dbuffer[dbindex].id, 
+       dbuffer[dbindex].gc, x, y, width, height);
 }
 
 /**
@@ -826,12 +935,18 @@ SX11Window::bitfill (const SColor& bg, int x, int y,
 void
 SX11Window::bitline (const SColor& fg, int x, int y, int tox, int toy)
 {
-  if (gcFG != (unsigned long)  fg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
-    gcFG = (unsigned long)  fg.getValue();
+    fprintf (stderr, "SX11Window::bitline called without beginDoubleBuffer\n");
+    return;
   }
-  XDrawLine (impl->display, (Window) id, gc, x, y, tox, toy);
+  if (dbuffer[dbindex].gcFG != (unsigned long)  fg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, 
+         SX11Color(impl, fg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  fg.getValue();
+  }
+  XDrawLine (impl->display, (Drawable) dbuffer[dbindex].id, dbuffer[dbindex].gc, x, y, tox, toy);
 }
 
 /**
@@ -842,12 +957,17 @@ SX11Window::bitline (const SColor& fg, int x, int y, int tox, int toy)
 void
 SX11Window::bitpoint (const SColor& fg, int x, int y)
 {
-  if (gcFG != (unsigned long)  fg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
-    gcFG = (unsigned long)  fg.getValue();
+    fprintf (stderr, "SX11Window::bitpoint called without beginDoubleBuffer\n");
+    return;
   }
-  XDrawPoint (impl->display, (Window) id, gc, x, y);
+  if (dbuffer[dbindex].gcFG != (unsigned long)  fg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, SX11Color(impl, fg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  fg.getValue();
+  }
+  XDrawPoint (impl->display, (Drawable) dbuffer[dbindex].id, dbuffer[dbindex].gc, x, y);
 }
 
 void
@@ -855,10 +975,19 @@ SX11Window::bitpoints (const SColor& fg, const int* _x, const int* _y,
          unsigned int _size)
 {
   if (_size==0) return;
-  if (gcFG != (unsigned long)  fg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
-    gcFG = (unsigned long)  fg.getValue();
+    fprintf (stderr, "SX11Window::bitpoints called without beginDoubleBuffer\n");
+    return;
+  }
+  if (dbuffer[dbindex].gcFG != (unsigned long)  fg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, 
+       SX11Color(impl, fg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  fg.getValue();
+    // If I don't do XSync there is a point drawing error of 
+    // highlighted text on Matrox Video card with X64
+    // XSync (impl->display, False);
   }
   XPoint* p = new XPoint[_size];
   CHECK_NEW (p);
@@ -867,7 +996,8 @@ SX11Window::bitpoints (const SColor& fg, const int* _x, const int* _y,
      p[i].x = _x[i];
      p[i].y = _y[i];
   }
-  XDrawPoints (impl->display, (Window) id, gc, p, _size, CoordModeOrigin);
+  XDrawPoints (impl->display, (Drawable) dbuffer[dbindex].id, 
+      dbuffer[dbindex].gc, p, _size, CoordModeOrigin);
   delete p;
 }
 
@@ -876,7 +1006,8 @@ void
 SX11Window::getKeyboardFocus()
 {
   impl->getKeyboardFocus (this);
-  XSetInputFocus (impl->display, (Window) id, RevertToNone, CurrentTime);
+  XSetInputFocus (impl->display, (Window) dbuffer[0].id, 
+      RevertToNone, CurrentTime);
   XIC xic = inputMethod.getIC();
   if (xic)
   {
@@ -888,7 +1019,7 @@ SX11Window::getKeyboardFocus()
 long
 SX11Window::getID ()
 {
-  return id;
+  return dbuffer[0].id;
 }
 
 /**
@@ -897,11 +1028,17 @@ SX11Window::getID ()
 void
 SX11Window::setClippingArea (int _x, int _y, unsigned int _width, unsigned int _height)
 {
+  // called outside dbufer begin and end
+  if (dbuffer[dbindex].id == None)
+  {
+    clipRegion = SX11Region (_x, _y, _width, _height);
+    return;
+  }
   XRectangle rect;
   rect.x = _x; rect.y = _y; rect.width = _width; rect.height = _height;
-  XSetClipRectangles (impl->display, gc, 0, 0, &rect, 1, Unsorted);
-  XSetClipRectangles (impl->display, gc_draw_and, 0, 0, &rect, 1, Unsorted);
-  XSetClipRectangles (impl->display, gc_draw_or, 0, 0, &rect, 1, Unsorted);
+  XSetClipRectangles (impl->display, dbuffer[dbindex].gc, 0, 0, &rect, 1, Unsorted);
+  XSetClipRectangles (impl->display, dbuffer[dbindex].gc_draw_and, 0, 0, &rect, 1, Unsorted);
+  XSetClipRectangles (impl->display, dbuffer[dbindex].gc_draw_or, 0, 0, &rect, 1, Unsorted);
   clipped = true;
 }
 
@@ -911,11 +1048,14 @@ SX11Window::setClippingArea (int _x, int _y, unsigned int _width, unsigned int _
 void
 SX11Window::removeClippingArea ()
 {
+  // called outside dbufer begin and end
+  if (dbuffer[dbindex].id == None) return;
+
   if (clipped)
   {
-     XSetClipMask (impl->display, gc, None);
-     XSetClipMask (impl->display, gc_draw_and, None);
-     XSetClipMask (impl->display, gc_draw_or, None);
+     XSetClipMask (impl->display, dbuffer[dbindex].gc, None);
+     XSetClipMask (impl->display, dbuffer[dbindex].gc_draw_and, None);
+     XSetClipMask (impl->display, dbuffer[dbindex].gc_draw_or, None);
      clipped = false;
   }
 }
@@ -937,7 +1077,10 @@ SX11Window::setApplicationImage (const SImage& im)
 {
   Pixmap pixmap = 0;
   Pixmap mask = 0;
+  unsigned int dbindexSave = dbindex;
+  dbindex = 0;
   createMaskedPixmap (im, &pixmap, &mask); 
+  dbindex = dbindexSave;
   //XWMHints *hints = XGetWMHints(impl->display, (Window) id);
   XWMHints *hints = XAllocWMHints();
   if (!pixmap ||!mask || !hints)
@@ -948,7 +1091,7 @@ SX11Window::setApplicationImage (const SImage& im)
   hints->icon_pixmap = pixmap;
   hints->flags |= IconMaskHint;
   hints->icon_mask = mask;
-  XSetWMHints(impl->display, (Window) id, hints); 
+  XSetWMHints(impl->display, (Window) dbuffer[0].id, hints); 
   XFree (hints);
 }
 
@@ -972,7 +1115,8 @@ SX11Window::redraw (bool clear, int _x, int _y, unsigned int _width, unsigned in
   {
     return;
   }
-  impl->addRedrawEvent (id, SRedrawEvent (clear, _x, _y, _width, _height)); 
+  impl->addRedrawEvent (dbuffer[0].id, 
+      SRedrawEvent (clear, _x, _y, _width, _height)); 
 }
 
 /**
@@ -982,7 +1126,7 @@ SX11Window::redraw (bool clear, int _x, int _y, unsigned int _width, unsigned in
  * @param properties provide some attributes to the input method.
  */
 bool
-SX11Window::startInputMethod (const SString& _name, const SProperties& prop)
+SX11Window::startInputMethod (const SString& _name, const SProperties& prop,  SPreEditor* preEditor)
 {
   return inputMethod.start (_name, prop);
 }
@@ -1013,6 +1157,11 @@ SX11Window::getInputMethod ()
   return SString(inputMethod.getName());
 }
 
+bool
+SX11Window::hasStatusArea () {
+  return inputMethod.hasStatusArea ();
+}
+
 bool
 SX11Window::isVisible ()
 {
@@ -1069,7 +1218,7 @@ SX11Window::bitpolygon (const SColor& fg, const SV_INT& poligon)
 SString
 SX11Window::getClipUTF8()
 {
-  return impl->getClipUTF8 (id);
+  return impl->getClipUTF8 (dbuffer[0].id);
 }
 
 /**
@@ -1078,7 +1227,7 @@ SX11Window::getClipUTF8()
 void
 SX11Window::putClipUTF8(const SString& utf8)
 {
-  impl->putClipUTF8 (id, utf8);
+  impl->putClipUTF8 (dbuffer[0].id, utf8);
 }
 
 /**
@@ -1092,7 +1241,7 @@ SX11Window::setMinimumSize (unsigned int _width, unsigned int _height)
   sizeHints.min_width = _width;
   sizeHints.min_height = _height;
   sizeHints.flags = PMinSize;
-  XSetWMProperties (impl->display, (Window) id, 0, 0, 0, 0, &sizeHints, 0, 0);
+  XSetWMProperties (impl->display, (Window) dbuffer[0].id, 0, 0, 0, 0, &sizeHints, 0, 0);
 }
 
 void
@@ -1104,14 +1253,19 @@ SX11Window::setTitle (const SString& title)
   XTextProperty   propertyIconName;
   XTextProperty   propertyWindowName;
 
-  XStoreName (impl->display, (Window) id, windowName.array());
+  XStoreName (impl->display, (Window) dbuffer[0].id, windowName.array());
 
   /* Discard const */
   char* wnc = (char*) windowName.array();
+#ifdef X_HAVE_UTF8_STRING
+  if (Xutf8TextListToTextProperty (impl->display, &wnc, 1, XUTF8StringStyle, &propertyWindowName)==Success &&
+     Xutf8TextListToTextProperty (impl->display, &wnc, 1, XUTF8StringStyle, &propertyIconName)==Success)
+#else
   if (XStringListToTextProperty (&wnc, 1, &propertyWindowName)!=0 &&
      XStringListToTextProperty (&wnc, 1, &propertyIconName)!=0)
+#endif
   {
-    XSetWMProperties (impl->display, (Window) id,
+    XSetWMProperties (impl->display, (Window) dbuffer[0].id,
        &propertyWindowName, &propertyIconName,
         0, 0, // argv, argc,
         0, 0, 0);
@@ -1119,7 +1273,7 @@ SX11Window::setTitle (const SString& title)
         XFree (propertyIconName.value);
   }
 
-  XSetWMProtocols (impl->display, (Window) id, &impl->wmDeleteWindow, 1);
+  XSetWMProtocols (impl->display, (Window) dbuffer[0].id, &impl->wmDeleteWindow, 1);
 }
 
 /**
@@ -1138,14 +1292,14 @@ void
 SX11Window::addAccelerator (const SAccelerator& a, SAcceleratorListener* l)
 {
   accelerators.put (a.toString(), l);
-  impl->addAccelerator (id, a);
+  impl->addAccelerator (dbuffer[0].id, a);
 }
 
 void
 SX11Window::removeAccelerator (const SAccelerator& a, SAcceleratorListener* l)
 {
   accelerators.remove (a.toString());
-  impl->removeAccelerator (id, a);
+  impl->removeAccelerator (dbuffer[0].id, a);
 }
 
 void
@@ -1178,7 +1332,7 @@ void
 SX11Window::setDroppable (const SStringVector& _targets)
 {
   const Atom xdndVersion = 3;
-  XChangeProperty (impl->display, (Window) id, 
+  XChangeProperty (impl->display, (Window) dbuffer[0].id, 
      XInternAtom (impl->display, "XdndAware", False), XA_ATOM, 32,
      PropModeReplace, (unsigned char*) &xdndVersion, 1);
   for (unsigned int i=0; i<_targets.size(); i++)
@@ -1204,15 +1358,15 @@ SX11Window::setModal (SWindow* _parent, bool decorated)
   winattr.save_under = True;
   if (_parent == 0)
   {
-    XSetTransientForHint (impl->display, (Window) id, (Window)impl->root);
+    XSetTransientForHint (impl->display, (Window) dbuffer[0].id, (Window)impl->root);
   }
   else
   {
-    XSetTransientForHint (impl->display, (Window) id, (Window)
-     (((SX11Window*)_parent)->id));
+    XSetTransientForHint (impl->display, (Window) dbuffer[0].id, (Window)
+     (((SX11Window*)_parent)->dbuffer[0].id));
   }
   if (decorated) return;
-  XChangeWindowAttributes (impl->display, (Window) id,
+  XChangeWindowAttributes (impl->display, (Window) dbuffer[0].id,
        CWOverrideRedirect | CWSaveUnder, &winattr);
 }
 /**
@@ -1224,7 +1378,7 @@ SX11Window::center (SWindow* _window)
   Window cw = impl->root;
   if (_window!=0)
   {
-    cw = (Window) ((SX11Window*)_window)->id;
+    cw = (Window) ((SX11Window*)_window)->dbuffer[0].id;
     cw = impl->getTopLevelWindow (cw);
     
   }
@@ -1277,12 +1431,16 @@ SX11Window::wait ()
 //  int lcount=0;
   impl->doXLoop();
   /* This is in a job and the job is removed already */
-  SEventHandler::addJob (impl->job, impl);
+  SJob* sjob = new SJob();
+  // SGC BUG in dialogs same job removed.
+  //SEventHandler::addJob (impl->job, impl);
+  SEventHandler::addJob (sjob, impl);
   while (shown && SEventHandler::next())
   {
 //    fprintf (stderr, "NEXT=%d\n", lcount++);
   }
-  SEventHandler::remove (impl->job);
+  SEventHandler::remove (sjob);
+  delete sjob;
 }
 
 XIC
@@ -1310,5 +1468,110 @@ SX11Window::isKInput ()
 unsigned long
 SX11Window::getWindowID() const
 {
-  return (unsigned long) id;
+  return (unsigned long) dbuffer[0].id;
+}
+
+void
+SX11Window::setDoubleBuffer (bool isOn)
+{
+  
+  dbindex = isOn ? 1 : 0;
+}
+
+bool
+SX11Window::isDoubleBufferEnabled () const
+{
+  return (dbindex == 1);
+}
+
+void
+SX11Window::freeDoubleBuffer ()
+{
+  if (dbuffer[1].id != None)
+  {
+    XFreeGC (impl->display, dbuffer[1].gc);
+    XFreeGC (impl->display, dbuffer[1].gc_default);
+    XFreeGC (impl->display, dbuffer[1].gc_draw_and);
+    XFreeGC (impl->display, dbuffer[1].gc_draw_or);
+    XFreePixmap (impl->display, (Pixmap) dbuffer[1].id);
+    dbuffer[1].id = None;
+  }
+}
+
+// called before global redraw to sync backing buffer with window.
+void
+SX11Window::beginDoubleBuffer (const SX11Region& region)
+{
+  if (dbindex == 0) return;
+  // Free Up Previous pixmap if 
+  if (dbuffer[1].id)
+  {
+    fprintf (stderr, "SX11Window::beginDoubleBuffer nested call?");
+    freeDoubleBuffer ();
+  }
+
+  dbuffer[1].gcFD = 0; // Font
+
+  unsigned int pw = (getWidth() == 0) ? 1 : getWidth();
+  unsigned int ph = (getHeight() == 0) ? 1 : getHeight();
+
+  int bits = DefaultDepth (impl->display, impl->screen);
+  dbuffer[1].id = XCreatePixmap (impl->display, (Window) dbuffer[0].id, 
+    pw,  ph, bits);
+
+  XGCValues   gcv;
+  gcv.foreground = SX11Color(impl, pen.getForeground()).getPixelValue();
+  gcv.background = background.getPixelValue();
+  clipped = false;
+  
+  dbuffer[1].gc = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground, &gcv); 
+  
+  dbuffer[1].gc_default = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+      0, &gcv);
+  
+  gcv.function = GXand;
+  dbuffer[1].gc_draw_and = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground | GCFunction, &gcv); 
+  
+  gcv.function = GXor;
+  dbuffer[1].gc_draw_or = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+       GCForeground | GCBackground | GCFunction, &gcv); 
+
+  XSetForeground (impl->display,  dbuffer[1].gc, 
+     SX11Color(impl, background).getPixelValue());
+  dbuffer[1].gcFG = (unsigned long)  background.getValue();
+
+  XFillRectangle (impl->display, (Drawable) dbuffer[1].id, 
+      dbuffer[1].gc, region.x, region.y, region.width,  region.height);
+  if (clipped)
+  {
+    XRectangle rect;
+    rect.x = clipRegion.x;
+    rect.y = clipRegion.y;
+    rect.width = clipRegion.width;
+    rect.height = clipRegion.height;
+    XSetClipRectangles (impl->display, dbuffer[1].gc, 0, 0, &rect, 1, Unsorted);
+    XSetClipRectangles (impl->display, dbuffer[1].gc_draw_and, 0, 0, &rect, 1, Unsorted);
+    XSetClipRectangles (impl->display, dbuffer[1].gc_draw_or, 0, 0, &rect, 1, Unsorted);
+  }
+}
+
+void
+SX11Window::endDoubleBuffer (const SX11Region& region)
+{
+  if (dbindex == 0)
+  {
+     if (dbuffer[1].id != None) freeDoubleBuffer ();
+     return;
+  }
+  if (dbuffer[1].id == 0)
+  {
+    fprintf (stderr, "SX11Window::endDoubleBuffer nested call?");
+    return;
+  }
+  XCopyArea (impl->display, (Drawable) dbuffer[1].id, 
+    (Drawable)dbuffer[0].id, dbuffer[dbindex].gc_default, region.x, region.y, 
+    region.width, region.height, region.x, region.y);
+  freeDoubleBuffer ();
 }
