@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -55,6 +55,7 @@ SXInputMethod::SXInputMethod (SX11Impl* _impl, long _id, long _eventMask)
   conversionOwner = None;
   xim = 0;
   xic = 0;
+  statusAreaOK = false;
   clientWindow = _id;
   isAscii = true;
   eventMask = _eventMask;
@@ -68,6 +69,12 @@ SXInputMethod::isAsciiInput ()
 {
   return isAscii;
 }
+
+bool
+SXInputMethod::hasStatusArea ()
+{
+  return statusAreaOK;
+}
 /**
  * return true if kinput is active and up
  */
@@ -120,11 +127,14 @@ SXInputMethod::start (const SString& _name, const SProperties& properties)
 
   name = "x-ascii";
   isAscii = true;
+  statusAreaOK = false;
 
-  if (_name.size() > 0 && _name[0] =='x' || _name[0] == 'X')
+  if (_name.size() > 0 && (_name[0] =='x' || _name[0] == 'X'))
   {
     return (createIC(_name, properties));
   }
+  statusAreaOK = true;
+
   /* switch off current one */
   if (oldName.size() > 1 && oldName[0] == 'x' && oldName[1] == '-'
    && oldName != "x-ascii")
@@ -157,11 +167,14 @@ SXInputMethod::start (const SString& _name, const SProperties& properties)
   unsigned long   nitems;
   unsigned long   bytesafter;
   unsigned long   *data;
+  unsigned char   *c_data=0;
 
   XGetWindowProperty (
     impl->display, conversionOwner, toAtom ("_CONVERSION_PROFILE"),
     0L, 100L, False, toAtom ("_CONVERSION_ATTRIBUTE_TYPE"),
-    &attributeType, &format, &nitems, &bytesafter, (unsigned char **)&data);
+    &attributeType, &format, &nitems, &bytesafter, &c_data);
+
+  data = (unsigned long*) c_data;
 
   if (data == 0) return false;
 
@@ -281,7 +294,7 @@ SXInputMethod::setProperties (const SProperties& props)
         XPoint spot; spot.x = x; spot.y = y;
 
         XVaNestedList attributes = XVaCreateNestedList (0, 
-          XNSpotLocation, &spot, 0);
+          XNSpotLocation, &spot, (char *) 0x0);
 
         if (attributes) 
         {
@@ -790,6 +803,7 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
 
 #ifdef HAVE_LOCALE
   SString zlocale = slocale;
+  if (zlocale.size() == 0) zlocale = "C";
   zlocale.append ((char)0);
   if (!setlocale (LC_ALL, zlocale.array()))
   {
@@ -953,6 +967,7 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
 #endif
   if (ps.get ("InputStyle") == 0) ps.put ("InputStyle", "none");
   SString s = ps["InputStyle"];
+  statusAreaOK = false;
 
   unsigned long code = XIMPreeditNone | XIMStatusNone;
   if (s=="preedit-root-status-root")
@@ -962,10 +977,12 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
   else if (s =="preedit-under-status-under")
   {
     code = XIMPreeditArea  | XIMStatusArea;
+    statusAreaOK = true;
   }
   else if (s =="preedit-over-status-under")
   {
     code = XIMPreeditPosition | XIMStatusArea;
+    statusAreaOK = true;
   }
   else if (s == "preedit-over-status-over")
   {
@@ -1013,12 +1030,13 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
     }
   }
   bool nostyle =  (i == xim_styles->count_styles);
-  if (nostyle && hasoverthespot && inputStyle != "None" & inputStyle != "none")
+  if (nostyle && hasoverthespot && inputStyle != "None" && inputStyle != "none")
   {
     //fprintf (stderr, "Reverting to preedit-over-status-over.\n");
     inputStyle = "preedit-over-status-over";
     code = XIMPreeditPosition  | XIMStatusNothing;
     nostyle = false;
+    statusAreaOK = false;
   }
   if (hasrootstyle && nostyle && inputStyle != "None" && inputStyle != "none")
   {
@@ -1026,6 +1044,7 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
     inputStyle = "preedit-root-status-root";
     code = XIMPreeditNothing  | XIMStatusNothing;
     nostyle = false;
+    statusAreaOK = false;
   }
   /* Try no precomposition style */
   if (nostyle && inputStyle == "none")
@@ -1033,11 +1052,12 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
     for (i=0; i<xim_styles->count_styles; i++)
     {
        if (xim_styles->supported_styles[i] 
-          == XIMPreeditNone|XIMStatusNone)
+          == (XIMPreeditNone|XIMStatusNone))
        {
           code = XIMPreeditNone|XIMStatusNone;
           inputStyle = "preedit-root-status-root";
           nostyle = false;
+          statusAreaOK = false;
           break;
        }
     }
@@ -1048,6 +1068,7 @@ SXInputMethod::createIC (const SString& str, const SProperties& props)
     fprintf (stderr, 
         "Style=%*.*s in %*.*s not found. Style negotiation failed.\n",  
         SSARGS(inputStyle), SSARGS(name));
+    statusAreaOK = false;
     XCloseIM(xim);
     xim = 0;
     return false;
@@ -1309,6 +1330,7 @@ SXInputMethod::getIC ()
   if ((xic == 0 || xim == 0) && !isAsciiInput())
   {
     createIC ("x-ascii", SProperties());
+fprintf (stderr, "x-ascii %lx", (unsigned long) xic);
   }
   return xic;
 }
